@@ -25,6 +25,7 @@ from functools import total_ordering
 from collections.abc import Mapping, Iterable
 
 from discopy import messages
+from discopy.utils import factory_name, from_tree, rsubs, rmap
 
 
 @total_ordering
@@ -59,8 +60,6 @@ class Ob:
 
     """
     def __init__(self, name):
-        if not str(name):
-            raise ValueError(messages.empty_name(name))
         self._name = name
 
     @property
@@ -71,7 +70,7 @@ class Ob:
         >>> x = Ob('x')
         >>> x.name
         'x'
-        >>> x.name = 'y'  # doctest: +ELLIPSIS
+        >>> x.name = 'y'
         Traceback (most recent call last):
         ...
         AttributeError: can't set attribute
@@ -95,6 +94,13 @@ class Ob:
     def __lt__(self, other):
         return self.name < other.name
 
+    def to_tree(self):
+        return {'factory': factory_name(self), 'name': self.name}
+
+    @classmethod
+    def from_tree(cls, tree):
+        return cls(tree['name'])
+
 
 class Arrow:
     """
@@ -237,12 +243,7 @@ class Arrow:
         return self.sum([self]) + other
 
     def __radd__(self, other):
-        return self.__add__(other)
-
-    @staticmethod
-    def sum(terms, dom=None, cod=None):
-        """ Formal sum of `terms`. """
-        return Sum(terms, dom, cod)
+        return self + other
 
     def then(self, *others):
         """
@@ -362,19 +363,19 @@ class Arrow:
 
         Parameters
         ----------
-        Either var, expr with:
-
         var : sympy.Symbol
             Subtituted variable.
         expr : sympy.Expr
             Substituting expression.
 
-        Or a list of such pairs for multiple substitution.
-
         Returns
         -------
         arrow : Arrow
 
+        Note
+        ----
+        You can give a list of (var, expr) pairs for multiple substitution.
+
         Examples
         --------
         >>> from sympy.abc import phi, psi
@@ -388,6 +389,55 @@ class Arrow:
         return self.upgrade(
             Functor(ob=lambda x: x, ar=lambda f: f.subs(*args))(self))
 
+    def lambdify(self, *symbols, **kwargs):
+        """
+        Turns a symbolic diagram into a function from parameters to diagram.
+
+        Parameters
+        ----------
+        symbols : list of sympy.Symbol
+            Inputs of the lambda.
+        kwargs : any
+            Passed to sympy.lambdify
+
+        Returns
+        -------
+        lambda : callable
+            Takes concrete values returns concrete diagrams.
+
+        Examples
+        --------
+        >>> from sympy.abc import phi, psi
+        >>> x, y, z = Ob('x'), Ob('y'), Ob('z')
+        >>> f, g = Box('f', x, y, data=phi), Box('g', y, z, data=psi)
+        >>> assert f.lambdify(psi)(42) == f
+        >>> assert (f >> g).lambdify(phi, psi)(42, 43)\\
+        ...     == Box('f', x, y, data=42) >> Box('g', y, z, data=43)
+        """
+        return lambda *xs: self.id(self.dom).then(*(
+            box.lambdify(*symbols, **kwargs)(*xs) for box in self.boxes))
+
+    def bubble(self, **params):
+        """ Returns a :class:`cat.Bubble` with the diagram inside. """
+        return self.bubble_factory(self, **params)
+
+    def fmap(self, func):
+        return func(self)
+
+    def to_tree(self):
+        """ Encodes an arrow as a tree. """
+        return {
+            'factory': factory_name(self),
+            'dom': self.dom.to_tree(), 'cod': self.cod.to_tree(),
+            'boxes': [box.to_tree() for box in self.boxes]}
+
+    @classmethod
+    def from_tree(cls, tree):
+        """ Decodes a tree as an arrow. """
+        dom, cod = map(from_tree, (tree['dom'], tree['cod']))
+        boxes = list(map(from_tree, tree['boxes']))
+        return cls(dom, cod, boxes, _scan=False)
+
 
 class Id(Arrow):
     """
@@ -417,6 +467,8 @@ class Id(Arrow):
     def __str__(self):
         return "Id({})".format(str(self.dom))
 
+    from_tree = Arrow.from_tree
+
 
 class AxiomError(Exception):
     """
@@ -449,11 +501,17 @@ class Box(Arrow):
     >>> assert f[:0] == Id(f.dom) and f[1:] == Id(f.cod)
 
     """
-    def __init__(self, name, dom, cod, data=None, _dagger=False):
-        """
-        """
-        if not str(name):
-            raise ValueError(messages.empty_name(name))
+    def __init__(self, name, dom, cod, **params):
+        def recursive_free_symbols(data):
+            if isinstance(data, Mapping):
+                data = data.values()
+            if isinstance(data, Iterable):
+                # Handles numpy 0-d arrays, which are actually not iterable.
+                if not hasattr(data, "shape") or data.shape != ():
+                    return set().union(*map(recursive_free_symbols, data))
+            return data.free_symbols if hasattr(data, "free_symbols") else {}
+        data, _dagger = params.get("data", None), params.get("_dagger", False)
+        self._free_symbols = recursive_free_symbols(data)
         self._name, self._dom, self._cod = name, dom, cod
         self._boxes, self._dagger, self._data = [self], _dagger, data
         Arrow.__init__(self, dom, cod, [self], _scan=False)
@@ -490,30 +548,23 @@ class Box(Arrow):
 
     @property
     def free_symbols(self):
-        def recursive_free_symbols(data):
-            if isinstance(data, Mapping):
-                return sum(map(recursive_free_symbols, data.values()), [])
-            if isinstance(data, Iterable):
-                return sum(map(recursive_free_symbols, data), [])
-            if hasattr(data, "free_symbols"):
-                return list(data.free_symbols)
-            return []
-        return set(recursive_free_symbols(self.data))
+        return self._free_symbols
 
     def subs(self, *args):
         if not any(var in self.free_symbols for var in (
                 {var for var, _ in args[0]} if len(args) == 1 else {args[0]})):
             return self
+        return type(self)(
+            self.name, self.dom, self.cod, _dagger=self._dagger,
+            data=rsubs(self.data, *args))
 
-        def recursive_subs(data, *args):
-            if isinstance(data, Mapping):
-                return {key: recursive_subs(value, *args)
-                        for key, value in data.items()}
-            if isinstance(data, Iterable):
-                return [recursive_subs(elem, *args) for elem in data]
-            return getattr(data, "subs", lambda *_: data)(*args)
-        return Box(self.name, self.dom, self.cod, _dagger=self._dagger,
-                   data=recursive_subs(self.data, *args))
+    def lambdify(self, *symbols, **kwargs):
+        if not any(x in self.free_symbols for x in symbols):
+            return lambda *xs: self
+        from sympy import lambdify
+        return lambda *xs: type(self)(
+            self.name, self.dom, self.cod, _dagger=self._dagger,
+            data=lambdify(symbols, self.data, **kwargs)(*xs))
 
     @property
     def is_dagger(self):
@@ -547,8 +598,9 @@ class Box(Arrow):
 
     def __eq__(self, other):
         if isinstance(other, Box):
-            return all(self.__getattribute__(x) == other.__getattribute__(x)
-                       for x in ['_name', 'dom', 'cod', 'data', '_dagger'])
+            attributes = ['_name', '_dom', '_cod', '_data', '_dagger']
+            return all(
+                getattr(self, x) == getattr(other, x) for x in attributes)
         if isinstance(other, Arrow):
             return len(other) == 1 and other[0] == self
         return False
@@ -556,6 +608,30 @@ class Box(Arrow):
     def __lt__(self, other):
         return self.name < other.name
 
+    def __call__(self, *args, **kwargs):
+        if hasattr(self, "_apply"):
+            return self._apply(self, *args, **kwargs)
+        raise TypeError("Box is not callable, try drawing.diagramize.")
+
+    def to_tree(self):
+        tree = {
+            'factory': factory_name(self),
+            'name': self.name,
+            'dom': self.dom.to_tree(),
+            'cod': self.cod.to_tree()}
+        if self.is_dagger:
+            tree['is_dagger'] = True
+        if self.data is not None:
+            tree['data'] = self.data
+        return tree
+
+    @classmethod
+    def from_tree(cls, tree):
+        name = tree['name']
+        dom, cod = map(from_tree, (tree['dom'], tree['cod']))
+        data, _dagger = tree.get('data', None), 'is_dagger' in tree
+        return cls(name=name, dom=dom, cod=cod, data=data, _dagger=_dagger)
+
 
 class Sum(Box):
     """
@@ -656,10 +732,80 @@ class Sum(Box):
         unit = Sum([], self.cod, self.dom)
         return self.upgrade(sum([f.dagger() for f in self.terms], unit))
 
+    @property
+    def free_symbols(self):
+        return {x for box in self.terms for x in box.free_symbols}
+
     def subs(self, *args):
         unit = Sum([], self.dom, self.cod)
         return self.upgrade(sum([f.subs(*args) for f in self.terms], unit))
 
+    def lambdify(self, *symbols, **kwargs):
+        return lambda *xs: self.sum(
+            [box.lambdify(*symbols, **kwargs)(*xs) for box in self.terms])
+
+    @staticmethod
+    def fmap(func):
+        def sum_func(diagram):
+            return type(diagram)([func(term) for term in diagram.terms])
+        return sum_func
+
+    def to_tree(self):
+        return {
+            'factory': factory_name(self),
+            'terms': [t.to_tree() for t in self.terms],
+            'dom': self.dom.to_tree(),
+            'cod': self.cod.to_tree()}
+
+    @classmethod
+    def from_tree(cls, tree):
+        dom, cod = map(from_tree, (tree['dom'], tree['cod']))
+        terms = list(map(from_tree, tree['terms']))
+        return cls(terms=terms, dom=dom, cod=cod)
+
+
+class Bubble(Box):
+    """ A unary operator on homsets. """
+    def __init__(self, inside, dom=None, cod=None):
+        dom = inside.dom if dom is None else dom
+        cod = inside.cod if cod is None else cod
+        self._inside = inside
+        Box.__init__(self, "Bubble", dom, cod)
+
+    @property
+    def inside(self):
+        """ The diagram inside a bubble. """
+        return self._inside
+
+    def __str__(self):
+        return "({}).bubble({})".format(
+            self.inside,
+            "" if (self.dom, self.cod) == (self.inside.dom, self.inside.cod)
+            else "dom={}, cod={}".format(self.dom, self.cod))
+
+    def __repr__(self):
+        return "Bubble({}{})".format(
+            repr(self.inside),
+            "" if (self.dom, self.cod) == (self.inside.dom, self.inside.cod)
+            else ", dom={}, cod={})".format(repr(self.dom), repr(self.cod)))
+
+    def to_tree(self):
+        return {
+            'factory': factory_name(self),
+            'inside': self.inside.to_tree(),
+            'dom': self.dom.to_tree(),
+            'cod': self.cod.to_tree()}
+
+    @classmethod
+    def from_tree(cls, tree):
+        dom, cod, inside = map(from_tree, (
+            tree['dom'], tree['cod'], tree['inside']))
+        return cls(dom=dom, cod=cod, inside=inside)
+
+
+Arrow.sum = Sum
+Arrow.bubble_factory = Bubble
+
 
 class Functor:
     """
@@ -702,6 +848,10 @@ class Functor:
     >>> assert F(f[::-1]) == F(f)[::-1]
     >>> assert F(f.dom) == F(f).dom and F(f.cod) == F(f).cod
 
+    Functors are bubble-preserving.
+
+    >>> assert F(f.bubble()) == F(f).bubble()
+
     See Also
     --------
     Quiver : For functors from infinitely-generated categories,
@@ -723,8 +873,7 @@ class Functor:
         >>> F = Functor({Ob('x'): Ob('y')}, {})
         >>> assert F.ob == {Ob('x'): Ob('y')}
         """
-        return self._ob\
-            if hasattr(self._ob, "__getitem__") else Quiver(self._ob)
+        return self._ob if isinstance(self._ob, Mapping) else Quiver(self._ob)
 
     @property
     def ar(self):
@@ -748,6 +897,9 @@ class Functor:
         if isinstance(arrow, Sum):
             return self.ar_factory.sum(
                 list(map(self, arrow)), self(arrow.dom), self(arrow.cod))
+        if isinstance(arrow, Bubble):
+            return self(arrow.inside).bubble(
+                dom=self(arrow.dom), cod=self(arrow.cod))
         if isinstance(arrow, Ob):
             return self.ob[arrow]
         if isinstance(arrow, Box):
@@ -759,7 +911,7 @@ class Functor:
         raise TypeError(messages.type_err(Arrow, arrow))
 
 
-class Quiver:
+class Quiver(Mapping):
     """
     Wraps a function into an immutable dict-like object, used as input for a
     :class:`Functor`.
@@ -807,3 +959,14 @@ class Quiver:
 
     def __repr__(self):
         return "Quiver({})".format(repr(self._func))
+
+    def __len__(self):
+        """
+        >>> dict(Quiver(lambda x: x))   # doctest: +ELLIPSIS
+        Traceback (most recent call last):
+        ...
+        TypeError: Quivers have no length, you can't iterate them.
+        """
+        raise TypeError("Quivers have no length, you can't iterate them.")
+
+    __iter__ = __len__
