@@ -5,11 +5,6 @@ discopy error messages.
 """
 
 
-def empty_name(got):
-    """ Empty name error. """
-    return "Expected non-empty name, got {}.".format(repr(got))
-
-
 def type_err(expected, got):
     """ Type error. """
     return "Expected {}.{}, got {} of type {}.{} instead.".format(
@@ -42,13 +37,6 @@ def are_not_adjoints(left, right):
     return "{} and {} are not adjoints.".format(left, right)
 
 
-def wrong_adjunction(left, right, cup):
-    """ Wrong adjunction error. """
-    return "There is no {0}({2}, {3}) in a rigid category. "\
-           "Maybe you meant {1}({2}, {3})?".format(
-               "Cup" if cup else "Cap", "Cap" if cup else "Cup", left, right)
-
-
 def cup_vs_cups(left, right):
     """ Simple type error. """
     return "Cup can only witness adjunctions between simple types. "\
@@ -78,7 +66,8 @@ def missing_types_for_empty_sum():
 
 def expected_pregroup():
     """ pregroup.draw error. """
-    return "Expected a pregroup diagram of shape `word @ ... @ word >> cups`,"\
+    return "Expected a pregroup diagram of shape"\
+           "`word @ ... @ word >> cups_and_swaps`,"\
            " use diagram.draw() instead."
 
 
