@@ -29,15 +29,19 @@ We can check the axioms for dagger monoidal categories, up to interchanger.
 >>> assert (f0 @ f1)[::-1][::-1] == f0 @ f1
 >>> assert (f0 @ f1)[::-1].interchange(0, 1) == f0[::-1] @ f1[::-1]
 
-We can check the Eckerman-Hilton argument, up to interchanger.
+We can check the Eckmann-Hilton argument, up to interchanger.
 
 >>> s0, s1 = Box('s0', Ty(), Ty()), Box('s1', Ty(), Ty())
 >>> assert s0 @ s1 == s0 >> s1 == (s1 @ s0).interchange(0, 1)
 >>> assert s1 @ s0 == s1 >> s0 == (s0 @ s1).interchange(0, 1)
+
+.. image:: ../_static/imgs/EckmannHilton.gif
+    :align: center
 """
 
-from discopy import cat, messages, drawing
-from discopy.cat import Ob, Quiver, AxiomError
+from discopy import cat, messages, drawing, rewriting
+from discopy.cat import Ob
+from discopy.utils import factory_name, from_tree
 
 
 class Ty(Ob):
@@ -148,29 +152,31 @@ class Ty(Ob):
         >>> assert xs.count(x) == xs.count(x[0]) == xs.objects.count(Ob('x'))
         """
         obj, = obj if isinstance(obj, Ty) else (obj, )
-        return self.objects.count(obj)
+        return self._objects.count(obj)
 
     @staticmethod
     def upgrade(old):
-        """ Allows class inheritance for tensor and __getitem__ """
+        """ Allows class inheritance for tensor and __getitem__. """
         return old
 
+    def downgrade(self):
+        """ Downgrades to :class:`discopy.monoidal.Ty`. """
+        return Ty(*self)
+
     def __eq__(self, other):
-        if not isinstance(other, Ty):
-            return False
-        return self.objects == other.objects
+        return isinstance(other, Ty) and self._objects == other._objects
 
     def __hash__(self):
         return hash(repr(self))
 
     def __repr__(self):
-        return "Ty({})".format(', '.join(repr(x.name) for x in self.objects))
+        return "Ty({})".format(', '.join(repr(x.name) for x in self._objects))
 
     def __str__(self):
-        return ' @ '.join(map(str, self)) or 'Ty()'
+        return ' @ '.join(map(str, self._objects)) or 'Ty()'
 
     def __len__(self):
-        return len(self.objects)
+        return len(self._objects)
 
     def __iter__(self):
         for i in range(len(self)):
@@ -178,8 +184,8 @@ class Ty(Ob):
 
     def __getitem__(self, key):
         if isinstance(key, slice):
-            return self.upgrade(Ty(*self.objects[key]))
-        return self.objects[key]
+            return self.upgrade(Ty(*self._objects[key]))
+        return self._objects[key]
 
     def __matmul__(self, other):
         return self.tensor(other)
@@ -192,6 +198,27 @@ class Ty(Ob):
             result = result @ self
         return result
 
+    def to_tree(self):
+        return {
+            'factory': factory_name(self),
+            'objects': [x.to_tree() for x in self.objects]}
+
+    @classmethod
+    def from_tree(cls, tree):
+        return cls(*map(from_tree, tree['objects']))
+
+
+def types(names):
+    """ Transforms strings into instances of :class:`discopy.monoidal.Ty`.
+
+    Examples
+    --------
+    >>> x, y, z = types("x y z")
+    >>> x, y, z
+    (Ty('x'), Ty('y'), Ty('z'))
+    """
+    return list(map(Ty, names.split()))
+
 
 class PRO(Ty):
     """ Implements the objects of a PRO, i.e. a non-symmetric PROP.
@@ -255,9 +282,8 @@ class Layer(cat.Box):
     """
     def __init__(self, left, box, right):
         self._left, self._box, self._right = left, box, right
-        name = "Layer({}, {}, {})".format(left, box, right)
         dom, cod = left @ box.dom @ right, left @ box.cod @ right
-        super().__init__(name, dom, cod)
+        super().__init__("Layer", dom, cod)
 
     def __iter__(self):
         yield self._left
@@ -310,11 +336,23 @@ class Diagram(cat.Arrow):
     >>> f0, f1, g = Box('f0', x, y), Box('f1', z, w), Box('g', y @ w, y)
     >>> d = Diagram(x @ z, y, [f0, f1, g], [0, 1, 0])
     >>> assert d == f0 @ f1 >> g
+
+    >>> d.draw(figsize=(2, 2),
+    ...        path='docs/_static/imgs/monoidal/arrow-example.png')
+
+    .. image:: ../_static/imgs/monoidal/arrow-example.png
+        :align: center
     """
     @staticmethod
     def upgrade(old):
         return old
 
+    def downgrade(self):
+        """ Downcasting to :class:`discopy.monoidal.Diagram`. """
+        dom, cod = Ty(*self.dom), Ty(*self.cod)
+        boxes, offsets = [box.downgrade() for box in self.boxes], self.offsets
+        return Diagram(dom, cod, boxes, offsets)
+
     def __init__(self, dom, cod, boxes, offsets, layers=None):
         if not isinstance(dom, Ty):
             raise TypeError(messages.type_err(Ty, dom))
@@ -332,16 +370,22 @@ class Diagram(cat.Arrow):
                 left = layers.cod[:off] if layers else dom[:off]
                 right = layers.cod[off + len(box.dom):]\
                     if layers else dom[off + len(box.dom):]
-                layers = layers >> Layer(left, box, right)
+                layers = layers >> self.layer_factory(left, box, right)
             layers = layers >> cat.Id(cod)
         self._layers, self._offsets = layers, tuple(offsets)
         super().__init__(dom, cod, boxes, _scan=False)
 
+    def to_tree(self):
+        return dict(cat.Arrow.to_tree(self), offsets=self.offsets)
+
+    @classmethod
+    def from_tree(cls, tree):
+        arrow = cat.Arrow.from_tree(tree)
+        return cls(arrow.dom, arrow.cod, arrow.boxes, tree['offsets'])
+
     @property
     def offsets(self):
-        """
-        The offset of a box is the number of wires to its left.
-        """
+        """ The offset of a box is the number of wires to its left. """
         return list(self._offsets)
 
     @property
@@ -374,7 +418,7 @@ class Diagram(cat.Arrow):
                     self.offsets + other.offsets,
                     layers=self.layers >> other.layers))
 
-    def tensor(self, *others):
+    def tensor(self, other=None, *rest):
         """
         Returns the horizontal composition of 'self' with a diagram 'other'.
 
@@ -384,6 +428,13 @@ class Diagram(cat.Arrow):
         >>> f0, f1 = Box('f0', x, y), Box('f1', z, w)
         >>> assert f0 @ f1 == f0.tensor(f1) == f0 @ Id(z) >> Id(y) @ f1
 
+        >>> (f0 @ f1).draw(
+        ...     figsize=(2, 2),
+        ...     path='docs/_static/imgs/monoidal/tensor-example.png')
+
+        .. image:: ../_static/imgs/monoidal/tensor-example.png
+            :align: center
+
         Parameters
         ----------
         other : :class:`Diagram`
@@ -393,11 +444,10 @@ class Diagram(cat.Arrow):
         diagram : :class:`Diagram`
             the tensor of 'self' and 'other'.
         """
-        if not others:
+        if other is None:
             return self
-        if len(others) > 1:
-            return self.tensor(others[0]).tensor(*others[1:])
-        other, = others
+        if rest:
+            return self.tensor(other).tensor(*rest)
         if isinstance(other, Sum):
             return self.sum([self]).tensor(other)
         if not isinstance(other, Diagram):
@@ -407,9 +457,9 @@ class Diagram(cat.Arrow):
         offsets = self.offsets + [n + len(self.cod) for n in other.offsets]
         layers = cat.Id(dom)
         for left, box, right in self.layers:
-            layers = layers >> Layer(left, box, right @ other.dom)
+            layers = layers >> self.layer_factory(left, box, right @ other.dom)
         for left, box, right in other.layers:
-            layers = layers >> Layer(self.cod @ left, box, right)
+            layers = layers >> self.layer_factory(self.cod @ left, box, right)
         return self.upgrade(Diagram(dom, cod, boxes, offsets, layers=layers))
 
     def __matmul__(self, other):
@@ -451,11 +501,17 @@ class Diagram(cat.Arrow):
         return self.id(left) @ box @ self.id(right)
 
     def subs(self, *args):
-        return self.upgrade(
-            Functor(ob=lambda x: x, ar=lambda f: f.subs(*args))(self))
+        return self.id(self.dom).then(*(
+            self.id(left) @ box.subs(*args) @ self.id(right)
+            for left, box, right in self.layers))
+
+    def lambdify(self, *symbols, **kwargs):
+        return lambda *xs: self.id(self.dom).then(*(
+            self.id(left) @ box.lambdify(*symbols, **kwargs)(*xs)
+            @ self.id(right) for left, box, right in self.layers))
 
     @staticmethod
-    def swap(left, right):
+    def swap(left, right, ar_factory=None, swap_factory=None):
         """
         Returns a diagram that swaps the left with the right wires.
 
@@ -471,10 +527,21 @@ class Diagram(cat.Arrow):
         diagram : monoidal.Diagram
             with :code:`diagram.dom == left @ right`
         """
-        return swap(left, right)
+        ar_factory = ar_factory or Diagram
+        swap_factory = swap_factory or Swap
+        if not left:
+            return ar_factory.id(right)
+        if len(left) == 1:
+            boxes = [
+                swap_factory(left, right[i: i + 1])
+                for i, _ in enumerate(right)]
+            offsets = range(len(right))
+            return ar_factory(left @ right, right @ left, boxes, offsets)
+        return ar_factory.id(left[:1]) @ ar_factory.swap(left[1:], right)\
+            >> ar_factory.swap(left[:1], right) @ ar_factory.id(left[1:])
 
     @staticmethod
-    def permutation(perm, dom=None):
+    def permutation(perm, dom=None, ar_factory=None):
         """
         Returns the diagram that encodes a permutation of wires.
 
@@ -490,346 +557,100 @@ class Diagram(cat.Arrow):
         -------
         diagram : monoidal.Diagram
         """
-        return permutation(perm, dom)
-
-    def interchange(self, i, j, left=False):
-        """
-        Returns a new diagram with boxes i and j interchanged.
-
-        Gets called recursively whenever :code:`i < j + 1 or j < i - 1`.
-
-        Parameters
-        ----------
-        i : int
-            Index of the box to interchange.
-        j : int
-            Index of the new position for the box.
-        left : bool, optional
-            Whether to apply left interchangers.
-
-        Notes
-        -----
-        By default, we apply only right exchange moves::
-
-            top >> Id(left @ box1.dom @ mid) @ box0 @ Id(right)
-                >> Id(left) @ box1 @ Id(mid @ box0.cod @ right) >> bottom
-
-        gets rewritten to::
-
-            top >> Id(left) @ box1 @ Id(mid @ box0.dom @ right)
-                >> Id(left @ box1.cod @ mid) @ box0 @ Id(right) >> bottom
-        """
-        if not 0 <= i < len(self) or not 0 <= j < len(self):
-            raise IndexError
-        if i == j:
-            return self
-        if j < i - 1:
-            result = self
-            for k in range(i - j):
-                result = result.interchange(i - k, i - k - 1, left=left)
-            return result
-        if j > i + 1:
-            result = self
-            for k in range(j - i):
-                result = result.interchange(i + k, i + k + 1, left=left)
-            return result
-        if j < i:
-            i, j = j, i
-        off0, off1 = self.offsets[i], self.offsets[j]
-        left0, box0, right0 = self.layers[i]
-        left1, box1, right1 = self.layers[j]
-        # By default, we check if box0 is to the right first, then to the left.
-        if left and off1 >= off0 + len(box0.cod):  # box0 left of box1
-            off1 = off1 - len(box0.cod) + len(box0.dom)
-            middle = left1[len(left0 @ box0.cod):]
-            layer0 = Layer(left0, box0, middle @ box1.cod @ right1)
-            layer1 = Layer(left0 @ box0.dom @ middle, box1, right1)
-        elif off0 >= off1 + len(box1.dom):  # box0 right of box1
-            off0 = off0 - len(box1.dom) + len(box1.cod)
-            middle = left0[len(left1 @ box1.dom):]
-            layer0 = Layer(left1 @ box1.cod @ middle, box0, right0)
-            layer1 = Layer(left1, box1, middle @ box0.dom @ right0)
-        elif off1 >= off0 + len(box0.cod):  # box0 left of box1
-            off1 = off1 - len(box0.cod) + len(box0.dom)
-            middle = left1[len(left0 @ box0.cod):]
-            layer0 = Layer(left0, box0, middle @ box1.cod @ right1)
-            layer1 = Layer(left0 @ box0.dom @ middle, box1, right1)
-        else:
-            raise InterchangerError(box0, box1)
-        boxes = self.boxes[:i] + [box1, box0] + self.boxes[i + 2:]
-        offsets = self.offsets[:i] + [off1, off0] + self.offsets[i + 2:]
-        layers = self.layers[:i] >> layer1 >> layer0 >> self.layers[i + 2:]
-        return self.upgrade(
-            Diagram(self.dom, self.cod, boxes, offsets, layers=layers))
-
-    def normalize(self, left=False):
-        """
-        Implements normalisation of connected diagrams, see arXiv:1804.07832.
-
-        Parameters
-        ----------
-        left : bool, optional
-            Passed to :meth:`Diagram.interchange`.
-
-        Yields
-        ------
-        diagram : :class:`Diagram`
-            Rewrite steps.
-
-        Examples
-        --------
-
-        >>> s0, s1 = Box('s0', Ty(), Ty()), Box('s1', Ty(), Ty())
-        >>> gen = (s0 @ s1).normalize()
-        >>> for _ in range(3): print(next(gen))
-        s1 >> s0
-        s0 >> s1
-        s1 >> s0
-        """
-        diagram = self
-        while True:
-            no_more_moves = True
-            for i in range(len(diagram) - 1):
-                box0, box1 = diagram.boxes[i], diagram.boxes[i + 1]
-                off0, off1 = diagram.offsets[i], diagram.offsets[i + 1]
-                if left and off1 >= off0 + len(box0.cod)\
-                        or not left and off0 >= off1 + len(box1.dom):
-                    diagram = diagram.interchange(i, i + 1, left=left)
-                    yield diagram
-                    no_more_moves = False
-            if no_more_moves:
-                break
-
-    def normal_form(self, normalize=None, **params):
-        """
-        Returns the normal form of a diagram.
-
-        Parameters
-        ----------
-        normalize : iterable of :class:`Diagram`, optional
-            Generator that yields rewrite steps, default is
-            :meth:`Diagram.normalize`.
-
-        params : any, optional
-            Passed to :code:`normalize`.
-
-        Raises
-        ------
-        NotImplementedError
-            Whenever :code:`normalize` yields the same rewrite steps twice.
-        """
-        diagram, cache = self, set()
-        for _diagram in (normalize or Diagram.normalize)(diagram, **params):
-            if _diagram in cache:
-                raise NotImplementedError(messages.is_not_connected(self))
-            diagram = _diagram
-            cache.add(diagram)
+        ar_factory = ar_factory or Diagram
+        if set(range(len(perm))) != set(perm):
+            raise ValueError("Input should be a permutation of range(n).")
+        if dom is None:
+            dom = PRO(len(perm))
+        if len(dom) != len(perm):
+            raise ValueError(
+                "Domain and permutation should have the same length.")
+        diagram = ar_factory.id(dom)
+        for i in range(len(dom)):
+            j = perm.index(i)
+            diagram = diagram >> ar_factory.id(diagram.cod[:i])\
+                @ ar_factory.swap(diagram.cod[i:j], diagram.cod[j:j + 1])\
+                @ ar_factory.id(diagram.cod[j + 1:])
+            perm = perm[:i] + [i] + perm[i:j] + perm[j + 1:]
         return diagram
 
-    def foliate(self, yield_slices=False):
+    def permute(self, *perm):
         """
-        Generator yielding the interchanger steps in the foliation of self.
-
-        Yields
-        ------
-        diagram : :class:`Diagram`
-            Rewrite steps of the foliation.
+        Returns :code:`self >> self.permutation(perm, self.dom)`.
 
         Parameters
         ----------
-        yield_slices : bool, optional
-            Yield the list of slices of self as last output,
-            used in :meth:`Diagram.foliation`.
+        perm : list of int
+            such that :code:`i` goes to :code:`perm[i]`
 
         Examples
         --------
-
-        >>> x, y = Ty('x'), Ty('y')
-        >>> f0, f1 = Box('f0', x, y), Box('f1', y, x)
-        >>> d = (f0 @ Id(y) >> f0.dagger() @ f1) @ (f0 >> f1)
-        >>> *_, slices = d.foliate(yield_slices=True)
-        >>> print(slices[0])
-        f0 @ Id(y @ x) >> Id(y) @ f1 @ Id(x) >> Id(y @ x) @ f0
-        >>> print(slices[1])
-        f0[::-1] @ Id(x @ y) >> Id(x @ x) @ f1
-        >>> ket = Box('ket', Ty(), x)
-        >>> scalar = Box('scalar', Ty(), Ty())
-        >>> kets = scalar @ ket @ scalar @ ket
-        >>> a = kets.foliate()
-        >>> assert next(a) == kets
+        >>> x, y, z = Ty('x'), Ty('y'), Ty('z')
+        >>> assert Id(x @ y @ z).permute(2, 1, 0).cod == z @ y @ x
         """
-        def is_right_of(last, diagram):
-            off0, off1 = diagram.offsets[last], diagram.offsets[last + 1]
-            box0, box1 = diagram.boxes[last], diagram.boxes[last + 1]
-            if off1 >= off0 + len(box0.cod):  # box1 right of box0
-                return True
-            if off0 >= off1 + len(box1.dom):  # box1 left of box0
-                return False
-            return None
+        return self >> self.permutation(list(perm), self.dom)
 
-        def move_in_slice(first, last, k, diagram):
-            result = diagram
-            try:
-                if not k == last + 1:
-                    result = diagram.interchange(k, last + 1)
-                right_of_last = is_right_of(last, result)
-                if right_of_last is None:
-                    return None
-                if right_of_last:
-                    return result
-                result = result.interchange(last + 1, last)
-                if last == first:
-                    return result
-                return move_in_slice(first, last - 1, last, result)
-            except InterchangerError:
-                return None
+    @staticmethod
+    def subclass(ar_factory):
+        """ Decorator for subclasses of Diagram. """
+        def upgrade(old):
+            ob_upgrade = type(ar_factory.id().dom).upgrade  # Is this Yoneda?
+            dom, cod = ob_upgrade(old.dom), ob_upgrade(old.cod)
+            return ar_factory(dom, cod, old.boxes, old.offsets, old.layers)
+        ar_factory.upgrade = staticmethod(upgrade)
+        return ar_factory
 
-        start, diagram = 0, self
-        if yield_slices:
-            slices = []
-        while start < len(diagram):
-            last = start
-            k = last + 1
-            while k < len(diagram):
-                result = move_in_slice(start, last, k, diagram)
-                k += 1
-                if result is None:
-                    pass
-                else:
-                    diagram = result
-                    last += 1
-                    yield diagram
-            if yield_slices:
-                slices += [diagram[start: last + 1]]
-            start = last + 1
-        if yield_slices:
-            yield slices
-
-    def flatten(self):
+    def open_bubbles(self):
         """
-        Takes a diagram of diagrams and returns a diagram.
+        Called when drawing bubbles. Replace each bubble by::
 
-        >>> x, y = Ty('x'), Ty('y')
-        >>> f0, f1 = Box('f0', x, y), Box('f1', y, x)
-        >>> g = Box('g', x @ y, y)
-        >>> d = (Id(y) @ f0 @ Id(x) >> f0.dagger() @ Id(y) @ f0 >>\\
-        ...      g @ f1 >> f1 @ Id(x)).normal_form()
-        >>> assert d.foliation().flatten().normal_form() == d
-        >>> assert d.foliation().dagger().flatten()\\
-        ...     == d.foliation().flatten().dagger()
+            open_bubble\\
+                >> Id(left) @ open_bubbles(bubble.inside) @ Id(right)\\
+                >> close_bubble
+
+        for :code:`left = Ty(bubble.drawing_name)` and :code:`right = Ty("")`.
+        :meth:`Diagram.downgrade` gets called in the process.
         """
-        return self.upgrade(
-            Functor(Quiver(lambda x: x), Quiver(lambda f: f))(self))
+        if not any(isinstance(box, Bubble) for box in self.boxes):
+            return self.downgrade()
 
-    def foliation(self):
-        """
-        Returns a diagram with normal_form diagrams of depth 1 as boxes
-        such that its flattening gives the original diagram back.
+        class OpenBubbles(Functor):
+            def __call__(self, diagram):
+                diagram = diagram.downgrade()
+                if isinstance(diagram, Bubble):
+                    obj = Ob(diagram.drawing_name)
+                    obj.draw_as_box = True
+                    left, right = Ty(obj), Ty("")
+                    open_bubble = Box(
+                        "open_bubble",
+                        diagram.dom, left @ diagram.inside.dom @ right)
+                    close_bubble = Box(
+                        "_close",
+                        left @ diagram.inside.cod @ right, diagram.cod)
+                    open_bubble.draw_as_wires = True
+                    close_bubble.draw_as_wires = True
+                    # Wires can go straight only if types have the same length.
+                    if len(diagram.dom) == len(diagram.inside.dom):
+                        open_bubble.bubble_opening = True
+                    if len(diagram.cod) == len(diagram.inside.cod):
+                        close_bubble.bubble_closing = True
+                    return open_bubble\
+                        >> Id(left) @ self(diagram.inside) @ Id(right)\
+                        >> close_bubble
+                return super().__call__(diagram)
+        return OpenBubbles(lambda x: x, lambda f: f)(self)
 
-        >>> x, y = Ty('x'), Ty('y')
-        >>> f0, f1 = Box('f0', x, y), Box('f1', y, x)
-        >>> d = f0 @ Id(y) >> f0.dagger() @ f1
-        >>> assert d.foliation().boxes[0] == f0 @ f1
-        >>> assert d.foliation().flatten().normal_form() == d
-        >>> assert d.foliation().flatten()\\
-        ...     == d[::-1].foliation()[::-1].flatten()\\
-        ...     == d[::-1].foliation().flatten()[::-1]
-        >>> assert d.foliation().flatten().foliation() == d.foliation()
-        >>> g = Box('g', x @ x, x @ y)
-        >>> diagram = (d >> g >> d) @ (d >> g >> d)
-        >>> slices = diagram.foliation()
-        >>> assert slices.boxes[0] == f0 @ f1 @ f0 @ f1
-        >>> *_, last_diagram = diagram.foliate()
-        >>> assert last_diagram == slices.flatten()
-        """
-        *_, slices = self.foliate(yield_slices=True)
-        return self.upgrade(
-            Diagram(self.dom, self.cod, slices, len(slices) * [0]))
-
-    def depth(self):
-        """
-        Computes the depth of a diagram by foliating it
-
-        >>> x, y = Ty('x'), Ty('y')
-        >>> f, g = Box('f', x, y), Box('g', y, x)
-        >>> assert Id(x @ y).depth() == 0
-        >>> assert f.depth() == 1
-        >>> assert (f @ g).depth() == 1
-        >>> assert (f >> g).depth() == 2
-        """
-        *_, slices = self.foliate(yield_slices=True)
-        return len(slices)
-
-    def width(self):
-        """
-        Computes the width of a diagram,
-        i.e. the maximum number of parallel wires.
-
-        >>> x = Ty('x')
-        >>> f = Box('f', x, x ** 4)
-        >>> assert (f @ Id(x ** 2) >> Id(x ** 2) @ f.dagger()).width() == 6
-        """
-        return max(len(self.dom), max(
-            len(left @ box.cod @ right) for left, box, right in self.layers))
-
-    def draw(self, **params):
-        """
-        Draws a diagram using networkx and matplotlib.
-
-        Parameters
-        ----------
-        draw_as_nodes : bool, optional
-            Whether to draw boxes as nodes, default is :code:`False`.
-        color : string, optional
-            Color of the box or node, default is white (:code:`'#ffffff'`) for
-            boxes and red (:code:`'#ff0000'`) for nodes.
-        textpad : pair of floats, optional
-            Padding between text and wires, default is :code:`(0.1, 0.1)`.
-        draw_types : bool, optional
-            Whether to draw type labels, default is :code:`False`.
-        draw_box_labels : bool, optional
-            Whether to draw box labels, default is :code:`True`.
-        aspect : string, optional
-            Aspect ratio, one of :code:`['auto', 'equal']`.
-        margins : tuple, optional
-            Margins, default is :code:`(0.05, 0.05)`.
-        fontsize : int, optional
-            Font size for the boxes, default is :code:`12`.
-        fontsize_types : int, optional
-            Font size for the types, default is :code:`12`.
-        figsize : tuple, optional
-            Figure size.
-        path : str, optional
-            Where to save the image, if `None` we call :code:`plt.show()`.
-        to_tikz : bool, optional
-            Whether to output tikz code instead of matplotlib.
-        """
-        return drawing.draw(self, **params)
-
-    def to_gif(self, *diagrams, **params):  # pragma: no cover
-        """
-        Builds a gif with the normalisation steps.
-
-        Parameters
-        ----------
-        diagrams : :class:`Diagram`, optional
-            Sequence of diagrams to draw.
-        path : str
-            Where to save the image, if :code:`None` a gif gets created.
-        timestep : int, optional
-            Time step in milliseconds, default is :code:`500`.
-        loop : bool, optional
-            Whether to loop, default is :code:`False`
-        params : any, optional
-            Passed to :meth:`Diagram.draw`.
-        """
-        return drawing.to_gif(self, *diagrams, **params)
-
-
-class InterchangerError(AxiomError):
-    """ This is raised when we try to interchange conected boxes. """
-    def __init__(self, box0, box1):
-        super().__init__("Boxes {} and {} do not commute.".format(box0, box1))
+    draw = drawing.draw
+    to_gif = drawing.to_gif
+    interchange = rewriting.interchange
+    normalize = rewriting.normalize
+    normal_form = rewriting.normal_form
+    foliate = rewriting.foliate
+    flatten = rewriting.flatten
+    foliation = rewriting.foliation
+    depth = rewriting.depth
+    width = rewriting.width
+    layer_factory = Layer
 
 
 class Id(cat.Id, Diagram):
@@ -839,28 +660,76 @@ class Id(cat.Id, Diagram):
     >>> f = Box('f', s, t)
     >>> assert f >> Id(t) == f == Id(s) >> f
     """
-    def __init__(self, dom):
+    def __init__(self, dom=Ty()):
         cat.Id.__init__(self, dom)
         Diagram.__init__(self, dom, dom, [], [], layers=cat.Id(dom))
 
+    from_tree = Diagram.from_tree
+
 
 Diagram.id = Id
 
 
 class Box(cat.Box, Diagram):
     """
-    Implements a box as a diagram with :code:`boxes=[self], offsets=[0]`.
+    A box is a diagram with :code:`boxes==[self]` and :code:`offsets==[0]`.
 
+    Parameters
+    ----------
+    name : any
+        Name of the box.
+    dom : :class:`discopy.monoidal.Ty`
+        Domain of the box.
+    cod : :class:`discopy.monoidal.Ty`
+        Codomain of the box.
+    data : any, optional
+        Extra data in the box.
+
+    Other parameters
+    ----------------
+
+    draw_as_spider : bool, optional
+        Whether to draw the box as a spider.
+    draw_as_wires : bool, optional
+        Whether to draw the box as wires, e.g. :class:`discopy.monoidal.Swap`.
+    drawing_name : str, optional
+        The name to use when drawing the box.
+    tikzstyle_name : str, optional
+        The name of the style when tikzing the box.
+    color : str, optional
+        The color to use when drawing the box, one of
+        :code:`"white", "red", "green", "blue", "yellow", "black"`.
+        Default is :code:`"red" if draw_as_spider else "white"`.
+    shape : str, optional
+        The shape to use when drawing a spider,
+        one of :code:`"circle", "rectangle"`.
+
+    Examples
+    --------
     >>> f = Box('f', Ty('x', 'y'), Ty('z'))
     >>> assert Id(Ty('x', 'y')) >> f == f == f >> Id(Ty('z'))
     >>> assert Id(Ty()) @ f == f == f @ Id(Ty())
     >>> assert f == f[::-1][::-1]
     """
-    def __init__(self, name, dom, cod, data=None, _dagger=False):
-        cat.Box.__init__(self, name, dom, cod, data=data, _dagger=_dagger)
-        layer = Layer(dom[0:0], self, dom[0:0])
+    def downgrade(self):
+        """ Downcasting to :class:`discopy.monoidal.Box`. """
+        box = Box.__new__(Box)
+        for attr, value in self.__dict__.items():
+            setattr(box, attr, value)
+        dom, cod = self.dom.downgrade(), self.cod.downgrade()
+        box._dom, box._cod, box._boxes = dom, cod, [box]
+        layer = Layer(box._dom[0:0], box, box._dom[0:0])
+        box._layers = cat.Arrow(dom, cod, [layer], _scan=False)
+        return box
+
+    def __init__(self, name, dom, cod, **params):
+        cat.Box.__init__(self, name, dom, cod, **params)
+        layer = self.layer_factory(dom[0:0], self, dom[0:0])
         layers = cat.Arrow(dom, cod, [layer], _scan=False)
         Diagram.__init__(self, dom, cod, [self], [0], layers=layers)
+        for attr, value in params.items():
+            if attr in drawing.ATTRIBUTES:
+                setattr(self, attr, value)
 
     def __eq__(self, other):
         if isinstance(other, Box):
@@ -874,7 +743,21 @@ class Box(cat.Box, Diagram):
         return hash(repr(self))
 
 
-class Swap(Box):
+class BinaryBoxConstructor:
+    """ Box constructor with left and right as input. """
+    def __init__(self, left, right):
+        self.left, self.right = left, right
+
+    def to_tree(self):
+        left, right = self.left.to_tree(), self.right.to_tree()
+        return dict(Box.to_tree(self), left=left, right=right)
+
+    @classmethod
+    def from_tree(cls, tree):
+        return cls(*map(from_tree, (tree['left'], tree['right'])))
+
+
+class Swap(BinaryBoxConstructor, Box):
     """
     Implements the symmetry of atomic types.
 
@@ -888,9 +771,11 @@ class Swap(Box):
     def __init__(self, left, right):
         if len(left) != 1 or len(right) != 1:
             raise ValueError(messages.swap_vs_swaps(left, right))
-        self.left, self.right, self.draw_as_wire = left, right, True
-        super().__init__(
-            "Swap({}, {})".format(left, right), left @ right, right @ left)
+        name, dom, cod =\
+            "Swap({}, {})".format(left, right), left @ right, right @ left
+        BinaryBoxConstructor.__init__(self, left, right)
+        Box.__init__(self, name, dom, cod)
+        self.draw_as_wires = True
 
     def __repr__(self):
         return "Swap({}, {})".format(repr(self.left), repr(self.right))
@@ -916,10 +801,47 @@ class Sum(cat.Sum, Box):
         return self.upgrade(sum(terms, unit))
 
     def draw(self, **params):
+        """ Drawing a sum as an equation with :code:`symbol='+'`. """
         return drawing.equation(*self.terms, symbol='+', **params)
 
 
+class Bubble(cat.Bubble, Box):
+    """
+    Bubble in a monoidal diagram, i.e. a unary operator on homsets.
+
+    Parameters
+    ----------
+    inside : discopy.monoidal.Diagram
+        The diagram inside the bubble.
+    dom : discopy.monoidal.Ty, optional
+        The domain of the bubble, default is :code:`inside.dom`.
+    cod : discopy.monoidal.Ty, optional
+        The codomain of the bubble, default is :code:`inside.cod`.
+
+    Examples
+    --------
+    >>> x, y = Ty('x'), Ty('y')
+    >>> f, g = Box('f', x, y ** 3), Box('g', y, y @ y)
+    >>> d = (f.bubble(dom=x @ x, cod=y) >> g).bubble()
+    >>> d.draw(path='docs/_static/imgs/monoidal/bubble-example.png')
+
+    .. image:: ../_static/imgs/monoidal/bubble-example.png
+        :align: center
+    """
+    def __init__(self, inside, dom=None, cod=None, **params):
+        self.drawing_name = params.get("drawing_name", "")
+        cat.Bubble.__init__(self, inside, dom, cod)
+        Box.__init__(self, self._name, self.dom, self.cod, data=self.data)
+
+    def downgrade(self):
+        """ Downcasting to :class:`discopy.monoidal.Bubble`. """
+        result = Bubble(self.inside.downgrade(), Ty(*self.dom), Ty(*self.cod))
+        result.drawing_name = self.drawing_name
+        return result
+
+
 Diagram.sum = Sum
+Diagram.bubble_factory = Bubble
 
 
 class Functor(cat.Functor):
@@ -935,6 +857,13 @@ class Functor(cat.Functor):
     >>> assert F(F(f0)) == f0
     >>> assert F(f0 @ f1) == f1 @ f0
     >>> assert F(f0 >> f0[::-1]) == f1 >> f1[::-1]
+    >>> source, target = f0 >> f0[::-1], F(f0 >> f0[::-1])
+    >>> drawing.equation(
+    ...     source, target, symbol='$\\\\mapsto$', figsize=(4, 2),
+    ...     path='docs/_static/imgs/monoidal/functor-example.png')
+
+    .. image:: ../_static/imgs/monoidal/functor-example.png
+        :align: center
     """
     def __init__(self, ob, ar, ob_factory=None, ar_factory=None):
         if ob_factory is None:
@@ -944,7 +873,7 @@ class Functor(cat.Functor):
         super().__init__(ob, ar, ob_factory=ob_factory, ar_factory=ar_factory)
 
     def __call__(self, diagram):
-        if isinstance(diagram, Sum):
+        if isinstance(diagram, (Sum, Bubble)):
             super().__call__(diagram)
         if isinstance(diagram, Ty):
             return self.ob_factory().tensor(*[
@@ -963,42 +892,3 @@ class Functor(cat.Functor):
                 scan = scan[:off] @ box.cod @ scan[off + len(box.dom):]
             return result
         raise TypeError(messages.type_err(Diagram, diagram))
-
-
-def swap(left, right, ar_factory=Diagram, swap_factory=Swap):
-    """ Constructs swap diagrams of arbitrary types """
-    if not left:
-        return ar_factory.id(right)
-    if len(left) == 1:
-        boxes = [
-            swap_factory(left, right[i: i + 1]) for i, _ in enumerate(right)]
-        offsets = range(len(right))
-        return ar_factory(left @ right, right @ left, boxes, offsets)
-    return ar_factory.id(left[:1]) @ ar_factory.swap(left[1:], right)\
-        >> ar_factory.swap(left[:1], right) @ ar_factory.id(left[1:])
-
-
-def permutation(perm, dom=None, ar_factory=Diagram):
-    """ Constructs permutation diagrams of arbitrary types """
-    if set(range(len(perm))) != set(perm):
-        raise ValueError("Input should be a permutation of range(n).")
-    if dom is None:
-        dom = PRO(len(perm))
-    if len(dom) != len(perm):
-        raise ValueError("Domain and permutation should have the same length.")
-    diagram = ar_factory.id(dom)
-    for i in range(len(dom)):
-        j = perm.index(i)
-        diagram = diagram >> ar_factory.id(diagram.cod[:i])\
-            @ ar_factory.swap(diagram.cod[i:j], diagram.cod[j:j + 1])\
-            @ ar_factory.id(diagram.cod[j + 1:])
-        perm = perm[:i] + [i] + perm[i:j] + perm[j + 1:]
-    return diagram
-
-
-def diagram_subclass(cls):
-    """ Decorator for subclasses of Diagram. """
-    def upgrade(old):
-        return cls(old.dom, old.cod, old.boxes, old.offsets, old.layers)
-    cls.upgrade = staticmethod(upgrade)
-    return cls
