@@ -11,9 +11,16 @@ The objects are given by the free pregroup, the arrows by planar diagrams.
 >>> assert t.l.r == t == t.r.l
 >>> left_snake, right_snake = Id(n.r).transpose(left=True), Id(n.l).transpose()
 >>> assert left_snake.normal_form() == Id(n) == right_snake.normal_form()
+>>> from discopy import drawing
+>>> drawing.equation(
+...     left_snake, Id(n), right_snake, figsize=(4, 2),
+...     path='docs/_static/imgs/rigid/snake-equation.png')
+
+.. image:: ../_static/imgs/rigid/snake-equation.png
+    :align: center
 """
 
-from discopy import cat, monoidal, messages
+from discopy import cat, monoidal, messages, rewriting
 from discopy.cat import AxiomError
 
 
@@ -63,6 +70,17 @@ class Ob(cat.Ob):
         return str(self.name) + (
             - self.z * '.l' if self.z < 0 else self.z * '.r')
 
+    def to_tree(self):
+        tree = super().to_tree()
+        if self.z:
+            tree['z'] = self.z
+        return tree
+
+    @classmethod
+    def from_tree(cls, tree):
+        name, z = tree['name'], tree.get('z', 0)
+        return cls(name=name, z=z)
+
 
 class Ty(monoidal.Ty, Ob):
     """ Implements pregroup types as lists of simple types.
@@ -127,30 +145,47 @@ class PRO(monoidal.PRO, Ty):
         return self
 
 
-@monoidal.diagram_subclass
+class Layer(monoidal.Layer):
+    @staticmethod
+    def upgrade(old):
+        return Layer(old._left, old._box, old._right)
+
+    @property
+    def l(self):
+        return Layer(self._right.l, self._box.l, self._left.l)
+
+    @property
+    def r(self):
+        return Layer(self._right.r, self._box.r, self._left.r)
+
+
+@monoidal.Diagram.subclass
 class Diagram(monoidal.Diagram):
     """ Implements diagrams in the free rigid monoidal category.
 
     >>> I, n, s = Ty(), Ty('n'), Ty('s')
     >>> Alice, jokes = Box('Alice', I, n), Box('jokes', I, n.r @ s)
     >>> boxes, offsets = [Alice, jokes, Cup(n, n.r)], [0, 1, 0]
-    >>> print(Diagram(Alice.dom @ jokes.dom, s, boxes, offsets))
+    >>> d = Diagram(Alice.dom @ jokes.dom, s, boxes, offsets)
+    >>> print(d)
     Alice >> Id(n) @ jokes >> Cup(n, n.r) @ Id(s)
+
+    >>> d.draw(figsize=(3, 2),
+    ...        path='docs/_static/imgs/rigid/diagram-example.png')
+
+    .. image:: ../_static/imgs/rigid/diagram-example.png
+        :align: center
     """
-    @staticmethod
-    def id(dom):
-        return Id(dom)
-
     @staticmethod
     def swap(left, right):
-        return monoidal.swap(
+        return monoidal.Diagram.swap(
             left, right, ar_factory=Diagram, swap_factory=Swap)
 
     @staticmethod
     def permutation(perm, dom=None):
         if dom is None:
             dom = PRO(len(perm))
-        return monoidal.permutation(perm, dom, ar_factory=Diagram)
+        return monoidal.Diagram.permutation(perm, dom, ar_factory=Diagram)
 
     def foliate(self, yield_slices=False):
         """
@@ -174,6 +209,12 @@ class Diagram(monoidal.Diagram):
         >>> assert Diagram.cups(a, a.r) == Cup(a, a.r)
         >>> assert Diagram.cups(a @ b, (a @ b).r) ==\\
         ...     Id(a) @ Cup(b, b.r) @ Id(a.r) >> Cup(a, a.r)
+
+        >>> Diagram.cups(a @ b, (a @ b).r).draw(figsize=(3, 1),\\
+        ... margins=(0.3, 0.05), path='docs/_static/imgs/rigid/cups.png')
+
+    .. image:: ../_static/imgs/rigid/cups.png
+        :align: center
         """
         return cups(left, right)
 
@@ -188,6 +229,11 @@ class Diagram(monoidal.Diagram):
         """
         return caps(left, right)
 
+    @staticmethod
+    def spiders(n_legs_in, n_legs_out, typ):
+        """ Constructs spiders with compound types."""
+        return spiders(n_legs_in, n_legs_out, typ)
+
     @staticmethod
     def fa(left, right):
         """ Forward application. """
@@ -197,7 +243,7 @@ class Diagram(monoidal.Diagram):
     @staticmethod
     def ba(left, right):
         """ Backward application. """
-        off = -len(left) or len(right)
+        off = len(left) or -len(right)
         return Diagram.cups(left, right[:off]) @ Id(right[off:])
 
     @staticmethod
@@ -205,6 +251,23 @@ class Diagram(monoidal.Diagram):
         """ Forward composition. """
         return Id(left) @ Diagram.cups(middle.l, middle) @ Id(right.l)
 
+    @staticmethod
+    def bc(left, middle, right):
+        """ Backward composition. """
+        return Id(left.r) @ Diagram.cups(middle, middle.r) @ Id(right)
+
+    @staticmethod
+    def fx(left, middle, right):
+        """ Forward crossed composition. """
+        return Id(left) @ Diagram.swap(middle.l, right.r) @ Id(middle) >>\
+            Diagram.swap(left, right.r) @ Diagram.cups(middle.l, middle)
+
+    @staticmethod
+    def bx(left, middle, right):
+        """ Backward crossed composition. """
+        return Id(middle) @ Diagram.swap(left.l, middle.r) @ Id(right) >>\
+            Diagram.cups(middle, middle.r) @ Diagram.swap(left.l, right)
+
     @staticmethod
     def curry(diagram, n_wires=1, left=False):
         """ Diagram currying. """
@@ -216,6 +279,51 @@ class Diagram(monoidal.Diagram):
         return Id(diagram.dom[:-n_wires]) @ Diagram.caps(wires, wires.l)\
             >> diagram @ Id(wires.l)
 
+    def _conjugate(self, use_left):
+        layers = self.layers
+        list_of_layers = []
+        for layer in layers._boxes:
+            layer_adj = layer.l if use_left else layer.r
+            left, box, right = layer_adj
+            list_of_layers += (Id(left) @ box @ Id(right)).layers.boxes
+
+        dom = layers.dom.l if use_left else layers.dom.r
+        cod = layers.cod.l if use_left else layers.cod.r
+        layers_adj = type(layers)(dom, cod, list_of_layers)
+        boxes_and_offsets = tuple(zip(*(
+            (box, len(left)) for left, box, _ in layers_adj))) or ([], [])
+        inputs = (dom, cod) + boxes_and_offsets
+        return self.upgrade(Diagram(*inputs, layers=layers_adj))
+
+    @property
+    def l(self):
+        return self._conjugate(use_left=True)
+
+    @property
+    def r(self):
+        return self._conjugate(use_left=False)
+
+    def dagger(self):
+        d = super().dagger()
+        d._layers._boxes = [Layer.upgrade(b) for b in d._layers._boxes]
+        return d
+
+    def transpose_box(self, i, left=False):
+        bend_left = left
+        layers = self.layers
+        if bend_left:
+            box_T = layers[i]._box.r.dagger().transpose(left=True)
+        else:
+            box_T = layers[i]._box.l.dagger().transpose(left=False)
+        left, _, right = layers[i]
+        layers_T = (Id(left) @ box_T @ Id(right)).layers.boxes
+        list_of_layers = layers._boxes[:i] + layers_T + layers._boxes[i + 1:]
+        layers = type(layers)(layers.dom, layers.cod, list_of_layers)
+        boxes_and_offsets = tuple(zip(*(
+            (box, len(left)) for left, box, _ in layers))) or ([], [])
+        inputs = (layers.dom, layers.cod) + boxes_and_offsets
+        return self.upgrade(Diagram(*inputs, layers=layers))
+
     def transpose(self, left=False):
         """
         >>> a, b = Ty('a'), Ty('b')
@@ -245,121 +353,22 @@ class Diagram(monoidal.Diagram):
             >> self.id(self.dom.r) @ self @ self.id(self.cod.r)\
             >> self.id(self.dom.r) @ self.cups(self.cod, self.cod.r)
 
-    def normalize(self, left=False):
-        """
-        Return a generator which yields normalization steps.
-
-        >>> n, s = Ty('n'), Ty('s')
-        >>> cup, cap = Cup(n, n.r), Cap(n.r, n)
-        >>> f, g, h = Box('f', n, n), Box('g', s @ n, n), Box('h', n, n @ s)
-        >>> diagram = g @ cap >> f[::-1] @ Id(n.r) @ f >> cup @ h
-        >>> for d in diagram.normalize(): print(d)  # doctest: +ELLIPSIS
-        g... >> Cup(n, n.r) @ Id(n)...
-        g >> f[::-1] >> Id(n) @ Cap(n.r, n) >> Cup(n, n.r) @ Id(n) >> f >> h
-        g >> f[::-1] >> f >> h
-        """
-        def follow_wire(diagram, i, j):
-            """
-            Given a diagram, the index of a box i and the offset j of an output
-            wire, returns (i, j, obstructions) where:
-            - i is the index of the box which takes this wire as input, or
-            len(diagram) if it is connected to the bottom boundary.
-            - j is the offset of the wire at its bottom end.
-            - obstructions is a pair of lists of indices for the boxes on
-            the left and right of the wire we followed.
-            """
-            left_obstruction, right_obstruction = [], []
-            while i < len(diagram) - 1:
-                i += 1
-                box, off = diagram.boxes[i], diagram.offsets[i]
-                if off <= j < off + len(box.dom):
-                    return i, j, (left_obstruction, right_obstruction)
-                if off <= j:
-                    j += len(box.cod) - len(box.dom)
-                    left_obstruction.append(i)
-                else:
-                    right_obstruction.append(i)
-            return len(diagram), j, (left_obstruction, right_obstruction)
-
-        def find_snake(diagram):
-            """
-            Given a diagram, returns (cup, cap, obstructions, left_snake)
-            if there is a yankable pair, otherwise returns None.
-            """
-            for cap in range(len(diagram)):
-                if not isinstance(diagram.boxes[cap], Cap):
-                    continue
-                for left_snake, wire in [(True, diagram.offsets[cap]),
-                                         (False, diagram.offsets[cap] + 1)]:
-                    cup, wire, obstructions =\
-                        follow_wire(diagram, cap, wire)
-                    not_yankable =\
-                        cup == len(diagram)\
-                        or not isinstance(diagram.boxes[cup], Cup)\
-                        or left_snake and diagram.offsets[cup] + 1 != wire\
-                        or not left_snake and diagram.offsets[cup] != wire
-                    if not_yankable:
-                        continue
-                    return cup, cap, obstructions, left_snake
-            return None
-
-        def unsnake(diagram, cup, cap, obstructions, left_snake=False):
-            """
-            Given a diagram and the indices for a cup and cap pair
-            and a pair of lists of obstructions on the left and right,
-            returns a new diagram with the snake removed.
-
-            A left snake is one of the form Id @ Cap >> Cup @ Id.
-            A right snake is one of the form Cap @ Id >> Id @ Cup.
-            """
-            left_obstruction, right_obstruction = obstructions
-            if left_snake:
-                for box in left_obstruction:
-                    diagram = diagram.interchange(box, cap)
-                    yield diagram
-                    for i, right_box in enumerate(right_obstruction):
-                        if right_box < box:
-                            right_obstruction[i] += 1
-                    cap += 1
-                for box in right_obstruction[::-1]:
-                    diagram = diagram.interchange(box, cup)
-                    yield diagram
-                    cup -= 1
-            else:
-                for box in left_obstruction[::-1]:
-                    diagram = diagram.interchange(box, cup)
-                    yield diagram
-                    for i, right_box in enumerate(right_obstruction):
-                        if right_box > box:
-                            right_obstruction[i] -= 1
-                    cup -= 1
-                for box in right_obstruction:
-                    diagram = diagram.interchange(box, cap)
-                    yield diagram
-                    cap += 1
-            boxes = diagram.boxes[:cap] + diagram.boxes[cup + 1:]
-            offsets = diagram.offsets[:cap] + diagram.offsets[cup + 1:]
-            layers = diagram.layers[:cap] >> diagram.layers[cup + 1:]
-            yield Diagram(diagram.dom, diagram.cod, boxes, offsets, layers)
-
-        diagram = self
-        while True:
-            yankable = find_snake(diagram)
-            if yankable is None:
-                break
-            for _diagram in unsnake(diagram, *yankable):
-                yield _diagram
-                diagram = _diagram
-        for _diagram in monoidal.Diagram.normalize(diagram, left=left):
-            yield _diagram
-
-    def normal_form(self, normalize=None, **params):
+    def normal_form(self, normalizer=None, **params):
         """
         Implements the normalisation of rigid monoidal categories,
         see arxiv:1601.05372, definition 2.12.
         """
         return super().normal_form(
-            normalize=normalize or Diagram.normalize, **params)
+            normalizer=normalizer or Diagram.normalize, **params)
+
+    normalize = rewriting.snake_removal
+    layer_factory = Layer
+
+
+Sum = cat.Sum
+
+Sum.l = property(cat.Sum.fmap(lambda d: d.l))
+Sum.r = property(cat.Sum.fmap(lambda d: d.r))
 
 
 class Id(monoidal.Id, Diagram):
@@ -368,11 +377,14 @@ class Id(monoidal.Id, Diagram):
     >>> t = Ty('a', 'b', 'c')
     >>> assert Id(t) == Diagram(t, t, [], [])
     """
-    def __init__(self, dom):
+    def __init__(self, dom=Ty()):
         monoidal.Id.__init__(self, dom)
         Diagram.__init__(self, dom, dom, [], [], layers=cat.Id(dom))
 
 
+Diagram.id = Id
+
+
 class Box(monoidal.Box, Diagram):
     """ Implements generators of rigid monoidal diagrams.
 
@@ -380,25 +392,63 @@ class Box(monoidal.Box, Diagram):
     >>> Box('f', a, b.l @ b, data={42})
     Box('f', Ty('a'), Ty(Ob('b', z=-1), 'b'), data={42})
     """
-    def __init__(self, name, dom, cod, data=None, _dagger=False):
-        monoidal.Box.__init__(self, name, dom, cod, data=data, _dagger=_dagger)
+    def __init__(self, name, dom, cod, **params):
+        monoidal.Box.__init__(self, name, dom, cod, **params)
         Diagram.__init__(self, dom, cod, [self], [0], layers=self.layers)
+        self._z = params.get("_z", 0)
+
+    def __eq__(self, other):
+        if isinstance(other, Box):
+            return self._z == other._z and monoidal.Box.__eq__(self, other)
+        if isinstance(other, Diagram):
+            return len(other) == 1 and other.boxes[0] == self\
+                and (other.dom, other.cod) == (self.dom, self.cod)
+        return False
+
+    def __hash__(self):
+        return hash(repr(self))
+
+    @property
+    def z(self):
+        return self._z
+
+    def dagger(self):
+        return type(self)(
+            name=self.name, dom=self.cod, cod=self.dom,
+            data=self.data, _dagger=not self._dagger, _z=self._z)
+
+    @property
+    def l(self):
+        return type(self)(
+            name=self.name, dom=self.dom.l, cod=self.cod.l,
+            data=self.data, _dagger=self._dagger, _z=self._z - 1)
+
+    @property
+    def r(self):
+        return type(self)(
+            name=self.name, dom=self.dom.r, cod=self.cod.r,
+            data=self.data, _dagger=self._dagger, _z=self._z + 1)
 
 
 class Swap(monoidal.Swap, Box):
     """ Implements swaps of basic types in a rigid category. """
     def __init__(self, left, right):
         monoidal.Swap.__init__(self, left, right)
-        dom, cod = left @ right, right @ left
-        Box.__init__(self, "Swap({}, {})".format(left, right), dom, cod)
+        Box.__init__(self, self.name, self.dom, self.cod)
 
 
-class Cup(Box):
+class Cup(monoidal.BinaryBoxConstructor, Box):
     """ Defines cups for simple types.
 
     >>> n = Ty('n')
     >>> Cup(n, n.r)
     Cup(Ty('n'), Ty(Ob('n', z=1)))
+
+    >>> Cup(n, n.r).draw(figsize=(2,1), margins=(0.5, 0.05),\\
+    ... path='docs/_static/imgs/rigid/cup.png')
+
+    .. image:: ../_static/imgs/rigid/cup.png
+        :align: center
     """
     def __init__(self, left, right):
         if not isinstance(left, Ty):
@@ -409,24 +459,38 @@ class Cup(Box):
             raise ValueError(messages.cup_vs_cups(left, right))
         if left.r != right and left != right.r:
             raise AxiomError(messages.are_not_adjoints(left, right))
-        if left == right.r:
-            raise AxiomError(messages.wrong_adjunction(left, right, cup=True))
-        self.left, self.right, self.draw_as_wire = left, right, True
-        super().__init__("Cup({}, {})".format(left, right), left @ right, Ty())
+        monoidal.BinaryBoxConstructor.__init__(self, left, right)
+        Box.__init__(
+            self, "Cup({}, {})".format(left, right), left @ right, Ty())
+        self.draw_as_wires = True
 
     def dagger(self):
         return Cap(self.left, self.right)
 
+    @property
+    def l(self):
+        return Cup(self.right.l, self.left.l)
+
+    @property
+    def r(self):
+        return Cup(self.right.r, self.left.r)
+
     def __repr__(self):
         return "Cup({}, {})".format(repr(self.left), repr(self.right))
 
 
-class Cap(Box):
+class Cap(monoidal.BinaryBoxConstructor, Box):
     """ Defines cups for simple types.
 
     >>> n = Ty('n')
     >>> Cap(n, n.l)
     Cap(Ty('n'), Ty(Ob('n', z=-1)))
+
+    >>> Cap(n, n.l).draw(figsize=(2,1), margins=(0.5, 0.05),\\
+    ... path='docs/_static/imgs/rigid/cap.png')
+
+    .. image:: ../_static/imgs/rigid/cap.png
+        :align: center
     """
     def __init__(self, left, right):
         if not isinstance(left, Ty):
@@ -437,18 +501,74 @@ class Cap(Box):
             raise ValueError(messages.cap_vs_caps(left, right))
         if left != right.r and left.r != right:
             raise AxiomError(messages.are_not_adjoints(left, right))
-        if left.r == right:
-            raise AxiomError(messages.wrong_adjunction(left, right, cup=False))
-        self.left, self.right, self.draw_as_wire = left, right, True
-        super().__init__("Cap({}, {})".format(left, right), Ty(), left @ right)
+        monoidal.BinaryBoxConstructor.__init__(self, left, right)
+        Box.__init__(
+            self, "Cap({}, {})".format(left, right), Ty(), left @ right)
+        self.draw_as_wires = True
 
     def dagger(self):
         return Cup(self.left, self.right)
 
+    @property
+    def l(self):
+        return Cap(self.right.l, self.left.l)
+
+    @property
+    def r(self):
+        return Cap(self.right.r, self.left.r)
+
     def __repr__(self):
         return "Cap({}, {})".format(repr(self.left), repr(self.right))
 
 
+class Spider(Box):
+    """
+    Spider box.
+
+    Parameters
+    ----------
+    n_legs_in, n_legs_out : int
+        Number of legs in and out.
+    typ : discopy.rigid.Ty
+        The type of the spider, needs to be atomic.
+
+    Examples
+    --------
+    >>> x = Ty('x')
+    >>> spider = Spider(1, 2, x)
+    >>> assert spider.dom == x and spider.cod == x @ x
+    """
+    def __init__(self, n_legs_in, n_legs_out, typ, **params):
+        self.typ = typ
+        if len(typ) > 1:
+            raise ValueError(
+                "Spider boxes can only have len(typ) == 1, "
+                "try Diagram.spiders instead.")
+        name = "Spider({}, {}, {})".format(n_legs_in, n_legs_out, typ)
+        dom, cod = typ ** n_legs_in, typ ** n_legs_out
+        cup_like = (n_legs_in, n_legs_out) in ((2, 0), (0, 2))
+        params = dict(dict(
+            draw_as_spider=not cup_like,
+            draw_as_wires=cup_like,
+            color="black", drawing_name=""), **params)
+        Box.__init__(self, name, dom, cod, **params)
+
+    def __repr__(self):
+        return "Spider({}, {}, {})".format(
+            len(self.dom), len(self.cod), repr(self.typ))
+
+    def dagger(self):
+        return type(self)(len(self.cod), len(self.dom), self.typ)
+
+    @property
+    def l(self):
+        return type(self)(len(self.dom), len(self.cod), self.typ.l)
+
+    @property
+    def r(self):
+        return type(self)(len(self.dom), len(self.cod), self.typ.r)
+
+
 class Functor(monoidal.Functor):
     """
     Implements rigid monoidal functors, i.e. preserving cups and caps.
@@ -464,6 +584,13 @@ class Functor(monoidal.Functor):
     >>> F = Functor(ob, ar)
     >>> sentence = Alice @ loves @ Bob >> Cup(n, n.r) @ Id(s) @ Cup(n.l, n)
     >>> assert F(sentence).normal_form() == Alice >> Id(n) @ Bob >> love_box
+    >>> from discopy import drawing
+    >>> drawing.equation(
+    ...     sentence, F(sentence), symbol='$\\\\mapsto$', figsize=(5, 2),
+    ...     path='docs/_static/imgs/rigid/functor-example.png')
+
+    .. image:: ../_static/imgs/rigid/functor-example.png
+        :align: center
     """
     def __init__(self, ob, ar, ob_factory=Ty, ar_factory=Diagram):
         super().__init__(ob, ar, ob_factory=ob_factory, ar_factory=ar_factory)
@@ -471,6 +598,8 @@ class Functor(monoidal.Functor):
     def __call__(self, diagram):
         if isinstance(diagram, monoidal.Ty):
             def adjoint(obj):
+                if not hasattr(obj, "z") or not obj.z:
+                    return self.ob[type(diagram)(obj)]
                 result = self.ob[type(diagram)(type(obj)(obj.name, z=0))]
                 if obj.z < 0:
                     for _ in range(-obj.z):
@@ -486,6 +615,19 @@ class Functor(monoidal.Functor):
         if isinstance(diagram, Cap):
             return self.ar_factory.caps(
                 self(diagram.cod[:1]), self(diagram.cod[1:]))
+        if isinstance(diagram, Spider):
+            return self.ar_factory.spiders(
+                len(diagram.dom), len(diagram.cod), self(diagram.typ))
+        if isinstance(diagram, Box):
+            if not hasattr(diagram, "z") or not diagram.z:
+                return super().__call__(diagram)
+            z = diagram.z
+            for _ in range(abs(z)):
+                diagram = diagram.l if z > 0 else diagram.r
+            result = super().__call__(diagram)
+            for _ in range(abs(z)):
+                result = result.l if z < 0 else result.r
+            return result
         if isinstance(diagram, monoidal.Diagram):
             return super().__call__(diagram)
         raise TypeError(messages.type_err(Diagram, diagram))
@@ -510,3 +652,24 @@ def cups(left, right, ar_factory=Diagram, cup_factory=Cup, reverse=False):
 def caps(left, right, ar_factory=Diagram, cap_factory=Cap):
     """ Constructs a diagram of nested caps. """
     return cups(left, right, ar_factory, cap_factory, reverse=True)
+
+
+def spiders(
+        n_legs_in, n_legs_out, typ,
+        ar_factory=Diagram, spider_factory=Spider):
+    """ Constructs a diagram of interleaving spiders. """
+    id, swap, spider = ar_factory.id, ar_factory.swap, spider_factory
+    ts = [typ[i:i + 1] for i in range(len(typ))]
+    result = id().tensor(*[spider(n_legs_in, n_legs_out, t) for t in ts])
+
+    for i, t in enumerate(ts):
+        for j in range(n_legs_in - 1):
+            result <<= id(result.dom[:i * j + i + j]) @ swap(
+                t, result.dom[i * j + i + j:i * n_legs_in + j]
+            ) @ id(result.dom[i * n_legs_in + j + 1:])
+
+        for j in range(n_legs_out - 1):
+            result >>= id(result.cod[:i * j + i + j]) @ swap(
+                result.cod[i * j + i + j:i * n_legs_out + j], t
+            ) @ id(result.cod[i * n_legs_out + j + 1:])
+    return result
