@@ -11,9 +11,10 @@ Implements dagger monoidal functors into tensors.
 >>> assert F(Alice >> loves >> Bob.dagger()) == 1
 """
 
-from discopy import messages, monoidal, rigid, config
+import numpy
+
+from discopy import cat, config, messages, monoidal, rigid
 from discopy.cat import AxiomError
-from discopy.monoidal import Sum
 from discopy.rigid import Ob, Ty, Cup, Cap
 
 
@@ -21,25 +22,15 @@ if config.IMPORT_JAX:  # pragma: no cover
     import warnings
     for msg in config.IGNORE_WARNINGS:
         warnings.filterwarnings("ignore", message=msg)
-    import jax.numpy as np
+    import jax
 
-    def array2string(array, max_length=config.NUMPY_THRESHOLD):
-        """ array2string is not implemented in jax.numpy """
-        flat = list(array)
-        flat = flat if len(flat) <= max_length else\
-            flat[:max_length // 2] + ["..."] + flat[1 - max_length // 2:]
-        return "[{}]".format(", ".join(map(str, flat)))
-    np.array2string = array2string
-else:
-    import numpy as np
-    from numpy import array2string as _array2string
-    np.set_printoptions(threshold=config.NUMPY_THRESHOLD)
+numpy.set_printoptions(threshold=config.NUMPY_THRESHOLD)
 
-    def array2string(array, **params):
-        """ makes sure we get the same doctest with numpy and jax.numpy """
-        return _array2string(array, **dict(params, separator=', '))\
-            .replace('[ ', '[').replace('  ', ' ')
-    np.array2string = array2string
+
+def array2string(array, **params):
+    """ Numpy array pretty print. """
+    return numpy.array2string(array, **dict(params, separator=', '))\
+        .replace('[ ', '[').replace('  ', ' ')
 
 
 class Dim(Ty):
@@ -54,12 +45,14 @@ class Dim(Ty):
         return Dim(*[x.name for x in old.objects])
 
     def __init__(self, *dims):
+        dims = map(lambda x: x if isinstance(x, monoidal.Ob) else Ob(x), dims)
+        dims = list(filter(lambda x: x.name != 1, dims))  # Dim(1) == Dim()
         for dim in dims:
-            if not isinstance(dim, int):
-                raise TypeError(messages.type_err(int, dim))
-            if dim < 1:
+            if not isinstance(dim.name, int):
+                raise TypeError(messages.type_err(int, dim.name))
+            if dim.name < 1:
                 raise ValueError
-        super().__init__(*[Ob(dim) for dim in dims if dim > 1])
+        super().__init__(*dims)
 
     def __getitem__(self, key):
         if isinstance(key, slice):
@@ -93,13 +86,45 @@ class Dim(Ty):
 class Tensor(rigid.Box):
     """ Implements a tensor with dom, cod and numpy array.
 
+    Examples
+    --------
     >>> m = Tensor(Dim(2), Dim(2), [0, 1, 1, 0])
     >>> v = Tensor(Dim(1), Dim(2), [0, 1])
     >>> v >> m >> v.dagger()
     Tensor(dom=Dim(1), cod=Dim(1), array=[0])
+
+    Notes
+    -----
+    Tensors can have sympy symbols as free variables.
+
+    >>> from sympy.abc import phi, psi
+    >>> v = Tensor(Dim(1), Dim(2), [phi, psi])
+    >>> d = v >> v.dagger()
+    >>> assert v >> v.dagger() == Tensor(
+    ...     Dim(1), Dim(1), [phi * phi.conjugate() + psi * psi.conjugate()])
+
+    These can be substituted and lambdifed.
+
+    >>> v.subs(phi, 0).lambdify(psi)(1)
+    Tensor(dom=Dim(1), cod=Dim(2), array=[0, 1])
+
+    We can also use jax.numpy by changing the class variable :code:`Tensor.np`.
+
+    >>> from contextlib import contextmanager
+    >>> import jax
+    >>> @contextmanager
+    ... def jaxify():
+    ...     Tensor.np, tmp = jax.numpy, Tensor.np
+    ...     yield
+    ...     Tensor.np = tmp
+    >>> with jaxify():
+    ...     f = lambda *xs: d.lambdify(phi, psi)(*xs).array[0]
+    ...     assert jax.grad(f)(1., 2.) == 2.
     """
+    np = jax.numpy if config.IMPORT_JAX else numpy
+
     def __init__(self, dom, cod, array):
-        self._array = np.array(array).reshape(dom @ cod or (1, ))
+        self._array = Tensor.np.array(array).reshape(tuple(dom @ cod) or (1, ))
         super().__init__("Tensor", dom, cod)
 
     def __iter__(self):
@@ -125,8 +150,7 @@ class Tensor(rigid.Box):
 
     def __repr__(self):
         return "Tensor(dom={}, cod={}, array={})".format(
-            self.dom, self.cod,
-            np.array2string(self.array.flatten()))
+            self.dom, self.cod, array2string(self.array.flatten()))
 
     def __str__(self):
         return repr(self)
@@ -145,9 +169,9 @@ class Tensor(rigid.Box):
 
     def __eq__(self, other):
         if not isinstance(other, Tensor):
-            return np.all(self.array == other)
+            return Tensor.np.all(self.array == other)
         return (self.dom, self.cod) == (other.dom, other.cod)\
-            and np.all(self.array == other.array)
+            and Tensor.np.all(self.array == other.array)
 
     def then(self, *others):
         if len(others) != 1 or any(isinstance(other, Sum) for other in others):
@@ -157,7 +181,7 @@ class Tensor(rigid.Box):
             raise TypeError(messages.type_err(Tensor, other))
         if self.cod != other.dom:
             raise AxiomError(messages.does_not_compose(self, other))
-        array = np.tensordot(self.array, other.array, len(self.cod))\
+        array = Tensor.np.tensordot(self.array, other.array, len(self.cod))\
             if self.array.shape and other.array.shape\
             else self.array * other.array
         return Tensor(self.dom, other.cod, array)
@@ -169,7 +193,7 @@ class Tensor(rigid.Box):
         if not isinstance(other, Tensor):
             raise TypeError(messages.type_err(Tensor, other))
         dom, cod = self.dom @ other.dom, self.cod @ other.cod
-        array = np.tensordot(self.array, other.array, 0)\
+        array = Tensor.np.tensordot(self.array, other.array, 0)\
             if self.array.shape and other.array.shape\
             else self.array * other.array
         source = range(len(dom @ cod))
@@ -177,18 +201,19 @@ class Tensor(rigid.Box):
             i if i < len(self.dom) or i >= len(self.dom @ self.cod @ other.dom)
             else i - len(self.cod) if i >= len(self.dom @ self.cod)
             else i + len(other.dom) for i in source]
-        return Tensor(dom, cod, np.moveaxis(array, source, target))
+        return Tensor(dom, cod, Tensor.np.moveaxis(array, source, target))
 
     def dagger(self):
-        array = np.moveaxis(
+        array = Tensor.np.moveaxis(
             self.array, range(len(self.dom @ self.cod)),
             [i + len(self.cod) if i < len(self.dom) else
              i - len(self.dom) for i in range(len(self.dom @ self.cod))])
-        return Tensor(self.cod, self.dom, np.conjugate(array))
+        return Tensor(self.cod, self.dom, Tensor.np.conjugate(array))
 
     @staticmethod
-    def id(dom):
-        return Tensor(dom, dom, np.identity(int(np.prod(dom))))
+    def id(dom=Dim(1)):
+        from numpy import prod
+        return Tensor(dom, dom, Tensor.np.identity(int(prod(dom))))
 
     @staticmethod
     def cups(left, right):
@@ -208,26 +233,44 @@ class Tensor(rigid.Box):
         target = [i + len(right) if i < len(left @ right @ left)
                   else i - len(left) for i in source]
         return Tensor(left @ right, right @ left,
-                      np.moveaxis(array, source, target))
+                      Tensor.np.moveaxis(array, source, target))
 
     def transpose(self, left=False):
         """
-        Returns the algebraic transpose.
+        Returns the diagrammatic transpose.
 
         Note
         ----
-        This is *not* the same as the diagrammatic transpose for complex dims.
+        This is *not* the same as the algebraic transpose for complex dims.
         """
         return Tensor(self.cod[::-1], self.dom[::-1], self.array.transpose())
 
     def conjugate(self):
-        """ Returns the conjugate of a tensor. """
-        return Tensor(self.dom, self.cod, np.conjugate(self.array))
+        """
+        Returns the diagrammatic conjugate of a tensor.
+
+        Note
+        ----
+        This is *not* the same as the algebraic conjugate for complex dims.
+        """
+        # reverse the wires for both inputs and outputs
+        dom, cod = self.dom, self.cod
+        array = Tensor.np.moveaxis(
+            self.array, range(len(dom @ cod)),
+            [len(dom) - i - 1 for i in range(len(dom @ cod))])
+        return Tensor(dom[::-1], cod[::-1], Tensor.np.conjugate(array))
+
+    l = r = property(conjugate)
 
     def round(self, decimals=0):
         """ Rounds the entries of a tensor up to a number of decimals. """
         return Tensor(self.dom, self.cod,
-                      np.around(self.array, decimals=decimals))
+                      Tensor.np.around(self.array, decimals=decimals))
+
+    def map(self, func):
+        """ Apply a function elementwise. """
+        return Tensor(
+            self.dom, self.cod, list(map(func, self.array.flatten())))
 
     @staticmethod
     def zeros(dom, cod):
@@ -239,20 +282,51 @@ class Tensor(rigid.Box):
         >>> assert Tensor.zeros(Dim(2), Dim(2))\\
         ...     == Tensor(Dim(2), Dim(2), [0, 0, 0, 0])
         """
-        return Tensor(dom, cod, np.zeros(dom @ cod))
+        return Tensor(dom, cod, Tensor.np.zeros(dom @ cod))
 
     def subs(self, *args):
-        array = [getattr(x, "subs", lambda y, *_: y)(*args)
-                 for x in self.array.flatten()]
-        return Tensor(self.dom, self.cod, array)
+        return self.map(lambda x: getattr(x, "subs", lambda y, *_: y)(*args))
 
-    def grad(self, *variables):  # pragma: no cover
-        """ Gradient with respect to vars. """
-        array = np.array([[
-            getattr(x, "diff", lambda _: 0)(var) for x in self.array.flatten()]
-            for var in variables])
-        array = array.reshape(Dim(len(variables)) @ self.dom @ self.cod)
-        return Tensor(Dim(len(variables)) @ self.dom, self.cod, array)
+    def grad(self, var, **params):
+        """ Gradient with respect to variables. """
+        return self.map(lambda x:
+                        getattr(x, "diff", lambda _: 0)(var, **params))
+
+    def jacobian(self, variables, **params):
+        """
+        Jacobian with respect to :code:`variables`.
+
+        Parameters
+        ----------
+        variables : List[sympy.Symbol]
+            Differentiated variables.
+
+        Returns
+        -------
+        tensor : Tensor
+            with :code:`tensor.dom == self.dom`
+            and :code:`tensor.cod == Dim(len(variables)) @ self.cod`.
+
+        Examples
+        --------
+        >>> from sympy.abc import x, y, z
+        >>> vector = Tensor(Dim(1), Dim(2), [x ** 2, y * z])
+        >>> vector.jacobian([x, y, z])
+        Tensor(dom=Dim(1), cod=Dim(3, 2), array=[2.0*x, 0, 0, 1.0*z, 0, 1.0*y])
+        """
+        dim = Dim(len(variables) or 1)
+        result = Tensor.zeros(self.dom, dim @ self.cod)
+        for i, var in enumerate(variables):
+            onehot = numpy.zeros(dim or (1, ))
+            onehot[i] = 1
+            result += Tensor(Dim(1), dim, onehot) @ self.grad(var)
+        return result
+
+    def lambdify(self, *symbols, **kwargs):
+        from sympy import lambdify
+        array = lambdify(
+            symbols, self.array, **dict({'modules': Tensor.np}, **kwargs))
+        return lambda *xs: Tensor(self.dom, self.cod, array(*xs))
 
 
 class Functor(rigid.Functor):
@@ -271,13 +345,21 @@ class Functor(rigid.Functor):
         return super().__repr__().replace("Functor", "tensor.Functor")
 
     def __call__(self, diagram):
+        if isinstance(diagram, Bubble):
+            return self(diagram.inside).map(diagram.func)
         if isinstance(diagram, monoidal.Sum):
             dom, cod = self(diagram.dom), self(diagram.cod)
             return sum(map(self, diagram), Tensor.zeros(dom, cod))
         if isinstance(diagram, monoidal.Ty):
             def obj_to_dim(obj):
-                result = self.ob[type(diagram)(obj.name)]
-                return result if isinstance(result, Dim) else Dim(result)
+                if isinstance(obj, rigid.Ob) and obj.z != 0:
+                    obj = type(obj)(obj.name)  # sets z=0
+                result = self.ob[type(diagram)(obj)]
+                if isinstance(result, int):
+                    result = Dim(result)
+                if not isinstance(result, Dim):
+                    result = Dim.upgrade(result)
+                return result
             return Dim(1).tensor(*map(obj_to_dim, diagram.objects))
         if isinstance(diagram, Cup):
             return Tensor.cups(self(diagram.dom[:1]), self(diagram.dom[1:]))
@@ -285,6 +367,10 @@ class Functor(rigid.Functor):
             return Tensor.caps(self(diagram.cod[:1]), self(diagram.cod[1:]))
         if isinstance(diagram, monoidal.Box)\
                 and not isinstance(diagram, monoidal.Swap):
+            if diagram.z % 2 != 0:
+                while diagram.z != 0:
+                    diagram = diagram.l if diagram.z > 0 else diagram.r
+                return self(diagram).conjugate()
             if diagram.is_dagger:
                 return self(diagram.dagger()).dagger()
             return Tensor(self(diagram.dom), self(diagram.cod),
@@ -304,23 +390,24 @@ class Functor(rigid.Functor):
                     i + dim(box.right)
                     if i < dim(diagram.dom @ scan[:off]) + dim(box.left)
                     else i - dim(box.left) for i in source]
-                array = np.moveaxis(array, list(source), list(target))
+                array = Tensor.np.moveaxis(array, list(source), list(target))
                 scan = scan[:off] @ box.cod @ scan[off + len(box.dom):]
                 continue
             left = dim(scan[:off])
             source = list(range(dim(diagram.dom) + left,
                                 dim(diagram.dom) + left + dim(box.dom)))
             target = list(range(dim(box.dom)))
-            array = np.tensordot(array, self(box).array, (source, target))
+            array =\
+                Tensor.np.tensordot(array, self(box).array, (source, target))
             source = range(len(array.shape) - dim(box.cod), len(array.shape))
             target = range(dim(diagram.dom) + left,
                            dim(diagram.dom) + left + dim(box.cod))
-            array = np.moveaxis(array, list(source), list(target))
+            array = Tensor.np.moveaxis(array, list(source), list(target))
             scan = scan[:off] @ box.cod @ scan[off + len(box.dom):]
         return Tensor(self(diagram.dom), self(diagram.cod), array)
 
 
-@monoidal.diagram_subclass
+@monoidal.Diagram.subclass
 class Diagram(rigid.Diagram):
     """
     Diagram with Tensor boxes.
@@ -354,6 +441,10 @@ class Diagram(rigid.Diagram):
         >>> import tensornetwork as tn
         >>> assert (vector >> vector[::-1]).eval(tn.contractors.auto) == 1
         """
+        if contractor is None and "numpy" not in Tensor.np.__package__:
+            raise Exception(
+                'Please provide a contractor when using a non-numpy backend.')
+
         if contractor is None:
             return Functor(ob=lambda x: x, ar=lambda f: f.array)(self)
         array = contractor(*self.to_tn()).tensor
@@ -373,6 +464,7 @@ class Diagram(rigid.Diagram):
 
         Examples
         --------
+        >>> import numpy as np
         >>> from tensornetwork import Node, Edge
         >>> vector = Box('vector', Dim(1), Dim(2), [0, 1])
         >>> nodes, output_edge_order = vector.to_tn()
@@ -381,7 +473,7 @@ class Diagram(rigid.Diagram):
         >>> assert output_edge_order == [node[0]]
         """
         import tensornetwork as tn
-        nodes = [tn.Node(np.eye(dim), 'input_{}'.format(i))
+        nodes = [tn.Node(Tensor.np.eye(dim), 'input_{}'.format(i))
                  for i, dim in enumerate(self.dom)]
         inputs, scan = [n[0] for n in nodes], [n[1] for n in nodes]
         for box, offset in zip(self.boxes, self.offsets):
@@ -399,7 +491,7 @@ class Diagram(rigid.Diagram):
     @staticmethod
     def cups(left, right):
         return rigid.cups(left, right, ar_factory=Diagram,
-                          cup_factory=lambda x, _: Frobenius(2, 0, x[0]))
+                          cup_factory=lambda x, _: Spider(2, 0, x[0]))
 
     @staticmethod
     def caps(left, right):
@@ -407,15 +499,92 @@ class Diagram(rigid.Diagram):
 
     @staticmethod
     def swap(left, right):
-        return monoidal.swap(
+        return monoidal.Diagram.swap(
             left, right, ar_factory=Diagram, swap_factory=Swap)
 
+    def grad(self, var, **params):
+        """ Gradient with respect to :code:`var`. """
+        if var not in self.free_symbols:
+            return self.sum([], self.dom, self.cod)
+        left, box, right, tail = tuple(self.layers[0]) + (self[1:], )
+        t1 = self.id(left) @ box.grad(var, **params) @ self.id(right) >> tail
+        t2 = self.id(left) @ box @ self.id(right) >> tail.grad(var, **params)
+        return t1 + t2
+
+    def jacobian(self, variables, **params):
+        """
+        Diagrammatic jacobian with respect to :code:`variables`.
+
+        Parameters
+        ----------
+        variables : List[sympy.Symbol]
+            Differentiated variables.
+
+        Returns
+        -------
+        tensor : Tensor
+            with :code:`tensor.dom == self.dom`
+            and :code:`tensor.cod == Dim(len(variables)) @ self.cod`.
+
+        Examples
+        --------
+        >>> from sympy.abc import x, y, z
+        >>> vector = Box("v", Dim(1), Dim(2), [x ** 2, y * z])
+        >>> vector.jacobian([x, y, z]).eval()
+        Tensor(dom=Dim(1), cod=Dim(3, 2), array=[2.0*x, 0, 0, 1.0*z, 0, 1.0*y])
+        """
+        dim = Dim(len(variables) or 1)
+        result = Sum([], self.dom, dim @ self.cod)
+        for i, var in enumerate(variables):
+            onehot = numpy.zeros(dim or (1, ))
+            onehot[i] = 1
+            result += Box(var, Dim(1), dim, onehot) @ self.grad(var)
+        return result
+
+    @staticmethod
+    def spiders(n_legs_in, n_legs_out, dim):
+        """
+        Spider diagram.
+
+        Parameters
+        ----------
+        n_legs_in, n_legs_out : int
+            Number of legs in and out.
+        dim : Dim
+            Dimension for each leg.
+
+        Examples
+        --------
+        >>> assert Diagram.spiders(3, 2, Dim(1)) == Id(Dim(1))
+        >>> assert Diagram.spiders(1, 2, Dim(2)) == Spider(1, 2, Dim(2))
+        >>> Diagram.spiders(1, 2, Dim(2, 3))  # doctest: +ELLIPSIS
+        Traceback (most recent call last):
+        ...
+        NotImplementedError
+        """
+        dim = dim if isinstance(dim, Dim) else Dim(dim)
+        if not dim:
+            return Id(dim)
+        if len(dim) == 1:
+            return Spider(n_legs_in, n_legs_out, dim)
+        raise NotImplementedError
+
 
 class Id(rigid.Id, Diagram):
     """ Identity tensor.Diagram """
+    def __init__(self, dom=Dim()):
+        rigid.Id.__init__(self, dom)
+        Diagram.__init__(self, dom, dom, [], [], layers=cat.Id(dom))
+
+
+class Sum(monoidal.Sum, Diagram):
+    """ Sums of tensor diagrams. """
+    def eval(self, contractor=None):
+        return sum(term.eval(contractor=contractor) for term in self.terms)
 
 
 Diagram.id = Id
+Diagram.sum = Sum
 
 
 class Swap(rigid.Swap, Diagram):
@@ -424,10 +593,20 @@ class Swap(rigid.Swap, Diagram):
 
 class Box(rigid.Box, Diagram):
     """ Box in a tensor.Diagram """
+    def __init__(self, name, dom, cod, data, **params):
+        rigid.Box.__init__(self, name, dom, cod, data=data, **params)
+        Diagram.__init__(self, dom, cod, [self], [0])
+
     @property
     def array(self):
         """ The array inside the box. """
-        return np.array(self.data).reshape(self.dom @ self.cod or (1, ))
+        dom, cod = self.dom, self.cod
+        return Tensor.np.array(self.data).reshape(tuple(dom @ cod) or (1, ))
+
+    def grad(self, var, **params):
+        return self.bubble(
+            func=lambda x: getattr(x, "diff", lambda _: 0)(var),
+            drawing_name="$\\partial {}$".format(var))
 
     def __repr__(self):
         return super().__repr__().replace("Box", "tensor.Box")
@@ -435,7 +614,7 @@ class Box(rigid.Box, Diagram):
     def __eq__(self, other):
         if not isinstance(other, Box):
             return False
-        return np.all(self.array == other.array)\
+        return Tensor.np.all(self.array == other.array)\
             and (self.name, self.dom, self.cod)\
             == (other.name, other.dom, other.cod)
 
@@ -444,36 +623,107 @@ class Box(rigid.Box, Diagram):
             (self.name, self.dom, self.cod, tuple(self.array.flatten())))
 
 
-class Frobenius(Box):
+class Spider(rigid.Spider, Box):
     """
-    Frobenius box.
+    Spider box.
 
     Parameters
     ----------
-    n_wires_in, n_wires_out : int
-        Number of input and output wires.
+    n_legs_in, n_legs_out : int
+        Number of legs in and out.
     dim : int
         Dimension for each leg.
 
     Examples
     --------
-    >>> vector = Box('vector', Dim(1), Dim(2), [0, 1])
-    >>> spider = Frobenius(1, 2, dim=2)
+    >>> vector = Box('vec', Dim(1), Dim(2), [0, 1])
+    >>> spider = Spider(1, 2, dim=2)
     >>> assert (vector >> spider).eval() == (vector @ vector).eval()
+    >>> from discopy import drawing
+    >>> drawing.equation(vector >> spider, vector @ vector, figsize=(3, 2),\\
+    ... path='docs/_static/imgs/tensor/frobenius-example.png')
+
+    .. image:: ../_static/imgs/tensor/frobenius-example.png
+        :align: center
     """
-    def __init__(self, n_wires_in, n_wires_out, dim):
-        import numpy
-        name = "Frobenius({}, {}, dim={})".format(n_wires_in, n_wires_out, dim)
-        dom, cod = Dim(dim) ** n_wires_in, Dim(dim) ** n_wires_out
-        array = numpy.zeros(dom @ cod)
-        for i in range(dim):
-            array[len(dom @ cod) * (i, )] = 1
-        self.draw_as_spider, self.color, self.drawing_name = True, "black", ""
+    def __init__(self, n_legs_in, n_legs_out, dim):
+        dim = dim if isinstance(dim, Dim) else Dim(dim)
+        rigid.Spider.__init__(self, n_legs_in, n_legs_out, dim)
+        array = numpy.zeros(self.dom @ self.cod)
+        for i in range(int(numpy.prod(dim))):
+            array[len(self.dom @ self.cod) * (i, )] = 1
+        Box.__init__(self, self.name, self.dom, self.cod, array)
         self.dim = dim
-        super().__init__(name, dom, cod, array)
 
-    def __repr__(self):
-        return self.name
 
-    def dagger(self):
-        return Frobenius(len(self.cod), len(self.dom), self.dim)
+class Bubble(monoidal.Bubble, Box):
+    """
+    Bubble in a tensor diagram, applies a function elementwise.
+
+    Parameters
+    ----------
+    inside : tensor.Diagram
+        The diagram inside the bubble.
+    func : callable
+        The function to apply, default is :code:`lambda x: int(not x)`.
+
+    Examples
+    --------
+
+    >>> men = Box("men", Dim(1), Dim(2), [0, 1])
+    >>> mortal = Box("mortal", Dim(2), Dim(1), [1, 1])
+    >>> men_are_mortal = (men >> mortal.bubble()).bubble()
+    >>> assert men_are_mortal.eval()
+    >>> men_are_mortal.draw(draw_type_labels=False,
+    ...                     path='docs/_static/imgs/tensor/men-are-mortal.png')
+
+    .. image:: ../_static/imgs/tensor/men-are-mortal.png
+        :align: center
+
+    >>> from sympy.abc import x
+    >>> f = Box('f', Dim(2), Dim(2), [1, 0, 0, x])
+    >>> g = Box('g', Dim(2), Dim(2), [-x, 0, 0, 1])
+    >>> def grad(diagram, var):
+    ...     return diagram.bubble(
+    ...         func=lambda x: getattr(x, "diff", lambda _: 0)(var),
+    ...         drawing_name="d${}$".format(var))
+    >>> lhs = grad(f >> g, x)
+    >>> rhs = (grad(f, x) >> g) + (f >> grad(g, x))
+    >>> assert lhs.eval() == rhs.eval()
+    >>> from discopy import drawing
+    >>> drawing.equation(lhs, rhs, figsize=(5, 2), draw_type_labels=False,
+    ...                  path='docs/_static/imgs/tensor/product-rule.png')
+
+    .. image:: ../_static/imgs/tensor/product-rule.png
+        :align: center
+    """
+    def __init__(self, inside, func=lambda x: int(not x), **params):
+        self.func = func
+        super().__init__(inside, **params)
+
+    def grad(self, var, **params):
+        """
+        The gradient of a bubble is given by the chain rule.
+
+        >>> from sympy.abc import x
+        >>> from discopy import drawing
+        >>> g = Box('g', Dim(2), Dim(2), [2 * x, 0, 0, x + 1])
+        >>> f = lambda d: d.bubble(func=lambda x: x ** 2, drawing_name="f")
+        >>> lhs, rhs = Box.grad(f(g), x), f(g).grad(x)
+        >>> drawing.equation(lhs, rhs, draw_type_labels=False,
+        ...                  path='docs/_static/imgs/tensor/chain-rule.png')
+
+        .. image:: ../_static/imgs/tensor/chain-rule.png
+            :align: center
+        """
+        from sympy import Symbol
+        tmp = Symbol("tmp")
+        name = "$\\frac{{\\partial {}}}{{\\partial {}}}$"
+        return Spider(1, 2, dim=self.dom)\
+            >> self.inside.bubble(
+                func=lambda x: self.func(tmp).diff(tmp).subs(tmp, x),
+                drawing_name=name.format(self.drawing_name, var))\
+            @ self.inside.grad(var) >> Spider(2, 1, dim=self.cod)
+
+
+Diagram.bubble_factory = Bubble
