@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 from pytest import raises
+from pprint import PrettyPrinter
 from discopy.cat import *
 
 
@@ -21,9 +22,6 @@ def test_Ob():
 
 def test_Ob_init():
     assert (Ob('x'), Ob(42), Ob('Alice')) == (Ob('x'), Ob(42), Ob('Alice'))
-    with raises(ValueError) as err:
-        Ob('')
-    assert str(err.value) == messages.empty_name('')
 
 
 def test_Ob_name():
@@ -171,8 +169,6 @@ def test_AxiomError():
 def test_Box():
     f = Box('f', Ob('x'), Ob('y'), data=[42, {0: 1}, lambda x: x])
     assert f >> Id(Ob('y')) == f == Id(Ob('x')) >> f
-    with raises(ValueError) as err:
-        Box('', '', '')
 
 
 def test_Box_dagger():
@@ -263,3 +259,35 @@ def test_total_ordering():
     assert sorted([z, y, x]) == [x, y, z]
     f, g = Box('f', x, y), Box('g', y, z)
     assert f < g
+
+
+def test_Bubble():
+    f = Box('f', Ob('x'), Ob('y'))
+    assert repr((f).bubble()) == "Bubble(Box('f', Ob('x'), Ob('y')))"
+    assert str(f.bubble()) == "(f).bubble()"
+
+
+def test_Box_call():
+    f = Box('f', Ob('x'), Ob('y'))
+    with raises(TypeError):
+        f(42)
+
+
+def test_fmap():
+    f = Box('f', Ob('x'), Ob('y'))
+    assert f.fmap(lambda x: x + x) == f + f
+
+
+def test_from_tree():
+    f = Box('f', Ob('x'), Ob('y'), data=[42, {0: 1}])
+    d = (f >> f[::-1].bubble()) + Id(Ob('x'))
+    assert from_tree(d.to_tree()) == d
+
+
+def test_sum_lambdify():
+    from sympy.abc import phi
+    f = Box('f', Ob('x'), Ob('y'), data=[phi])
+    g = Box('g', Ob('x'), Ob('y'), data=[phi])
+
+    assert (f + g).free_symbols == {phi}
+    assert (f + g).lambdify(phi)(1) == f.lambdify(phi)(1) + g.lambdify(phi)(1)
