@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
 
 from pytest import raises
+from discopy.cat import *
 from discopy.monoidal import *
+from discopy.rewriting import *
 
 
 def test_Ty():
