@@ -66,7 +66,8 @@ def test_Diagram_normal_form():
     assert f.normal_form() == f
     assert f.transpose().transpose(left=True).normal_form() == f
     assert f.transpose(left=True).transpose().normal_form() == f
-    diagram = f.transpose(left=True).transpose(left=True).transpose().transpose()
+    diagram = f\
+        .transpose(left=True).transpose(left=True).transpose().transpose()
     assert diagram.normal_form() == f
 
     Eckmann_Hilton = Box('s0', Ty(), Ty()) @ Box('s1', Ty(), Ty())
@@ -103,6 +104,20 @@ def test_Cap_init():
     assert str(err.value) == messages.cap_vs_caps(Ty(), Ty())
 
 
+def test_Cup_Cap_dagger():
+    n = Ty('n')
+    assert Cap(n, n.l).dagger() == Cup(n, n.l)
+    assert Cup(n, n.l).dagger() == Cap(n, n.l)
+
+
+def test_Cup_Cap_adjoint():
+    n = Ty('n')
+    assert Cap(n, n.l).l == Cap(n.l.l, n.l)
+    assert Cap(n, n.l).r == Cap(n, n.r)
+    assert Cup(n, n.r).l == Cup(n, n.l)
+    assert Cup(n, n.r).r == Cup(n.r.r, n.r)
+
+
 def test_AxiomError():
     n, s = Ty('n'), Ty('s')
     with raises(AxiomError) as err:
@@ -117,22 +132,19 @@ def test_AxiomError():
     with raises(AxiomError) as err:
         Cap(n, n.l.l)
     assert str(err.value) == messages.are_not_adjoints(n, n.l.l)
-    with raises(AxiomError):
-        Cap(n, n.r)
-    with raises(AxiomError):
-        Cap(n, n.l).dagger()
-    with raises(AxiomError):
-        Cup(n, n.l)
-    with raises(AxiomError):
-        Cup(n, n.r).dagger()
 
 
 def test_Functor_call():
     x = Ty('x')
     cup, cap = Cup(x, x.r), Cap(x.r, x)
-    F = Functor(lambda x: x @ x, {})
+    box1, box2 = Box('box', x, x), Box('box', x @ x, x @ x)
+    spider = Spider(0, 2, x)
+    F = Functor(lambda x: x @ x, {box1: box2})
     assert F(cup) == Id(x) @ cup @ Id(x.r) >> cup
     assert F(cap) == Id(x.r) @ cap @ Id(x) << cap
+    assert F(box1) == box2
+    assert F(box1.l) == box2.l and F(box1.r) == box2.r
+    assert F(spider) == spider @ spider >> Id(x) @ Swap(x, x) @ Id(x)
     with raises(TypeError):
         F(F)
 
@@ -141,3 +153,58 @@ def test_Diagram_permutation():
     assert Diagram.permutation([2, 0, 1])\
         == Diagram.swap(PRO(1), PRO(1)) @ Id(PRO(1))\
         >> Id(PRO(1)) @ Diagram.swap(PRO(1), PRO(1))
+
+
+def test_adjoint():
+    n, s = map(Ty, 'ns')
+    Bob = Box('Bob', Ty(), n)
+    eats = Box('eats', Ty(), n.r @ s)
+    Bob_l = Box('Bob', Ty(), n.l, _z=-1)
+    Bob_r = Box('Bob', Ty(), n.r, _z=1)
+    eats_l = Box('eats', Ty(), s.l @ n, _z=-1)
+    eats_r = Box('eats', Ty(), s.r @ n.r.r, _z=1)
+    assert Bob.l.z == -1 and Bob.z == 0 and Bob.r.z == 1
+    assert Bob.l == Bob_l and Bob.r == Bob_r
+    assert eats.l == eats_l and eats.r == eats_r
+    diagram = Bob @ eats >> Cup(n, n.r) @ Id(s)
+    assert diagram.l == Bob_l >> eats_l @ Id(n.l) >> Id(s.l) @ Cup(n, n.l)
+    assert diagram.r == Bob_r >> eats_r @ Id(n.r) >> Id(s.r) @ Cup(n.r.r, n.r)
+
+
+def test_transpose_box():
+    n = Ty('s')
+    Bob = Box('Bob', Ty(), n)
+    Bob_Tl = Box('Bob', n.l, Ty(), _z=-1, _dagger=True)
+    Bob_Tr = Box('Bob', n.r, Ty(), _z=1, _dagger=True)
+    Bob.transpose_box(0, left=True) == Cap(n.r, n) >> Bob_Tr @ Id(n)
+    Bob.transpose_box(0) == Cap(n, n.l) >> Id(n) @ Bob_Tl
+
+
+def test_sum_adjoint():
+    x = Ty('x')
+    two, boxes = Box('two', x, x), Box('boxes', x, x)
+    two_boxes = two + boxes
+    assert two_boxes.l == two.l + boxes.l
+    assert two_boxes.l.r == two_boxes
+
+
+def test_spider_adjoint():
+    n = Ty('n')
+    one = Box('one', Ty(), n)
+    two = Box('two', Ty(), n)
+    diagram = one @ two >> Spider(2, 1, n)
+
+    assert diagram.l == one.l >> two.l @ Id(n.l) >> Spider(2, 1, n.l)
+    assert diagram.r == one.r >> two.r @ Id(n.r) >> Spider(2, 1, n.r)
+
+
+def test_spider_factory():
+    a, b, c = map(Ty, 'abc')
+    ts = [a, a @ b, a @ b @ c]
+    for i in range(5):
+        for j in range(5):
+            for t in ts:
+                s = spiders(i, j, t)
+                for k, ob in enumerate(t):
+                    assert all(map(ob.__eq__, s.dom[k::len(t)]))
+                    assert all(map(ob.__eq__, s.cod[k::len(t)]))
